@@ -15,6 +15,10 @@ No proper C parsing is done here (pycparser sucked, unfortunately).
 
 import sys, re, os
 
+if '-h' in sys.argv or '--help' in sys.argv:
+  print(__doc__)
+  exit(0)
+
 def err(msg):
   sys.stderr.write(msg + '\n')
 
@@ -133,9 +137,11 @@ class Event:
     return cmp(event.name, other.name)
 
 class Edge:
-  def __init__(edge, to_state, event_name=None, style=None, action=None):
+  def __init__(edge, to_state, event_name=None, style=None, action=None, color=None, arrow_head=None):
     edge.to_state = to_state
     edge.style = style
+    edge.color = color
+    edge.arrow_head = arrow_head
     edge.events = []
     edge.actions = []
     edge.add_event_name(event_name)
@@ -187,6 +193,7 @@ class State:
   out_state_names = None
   out_edges = None
   kind = None
+  color = None
 
   def __init__(state):
     state.in_event_names = []
@@ -201,9 +208,11 @@ class State:
           out_edge.add_events(edge.events)
           out_edge.add_actions(edge.actions)
           return
-      # sanity
-      if out_edge.to_state.get_label() == edge.to_state.get_label():
-        raise Exception('Two distinct states exist with identical labels.')
+      elif out_edge.to_state.get_label() == edge.to_state.get_label():
+        # sanity: there already is an edge to a state that a) is not identical to the target state of the
+        # newly added edge but b) has the same label.
+        raise Exception('Two distinct states exist with identical label: %r: states %r and %r.'
+                        % (out_edge.to_state.get_label(), out_edge.to_state, edge.to_state))
     state.out_edges.append(edge)
 
   def get_label(state):
@@ -228,6 +237,11 @@ class State:
       return ''
     return ',shape=%s' % shape
 
+  def color_str(state):
+    if state.color is None:
+      return ''
+    return ',color="%s"' % state.color
+
   def __repr__(state):
     return 'State(name=%r,short_name=%r,out=%d)' % (state.name, state.short_name, len(state.out_edges))
 
@@ -242,7 +256,13 @@ class Fsm:
     fsm.event_names = set()
     fsm.dot_name = fsm.all_names_sanitized()
 
-  def parse_states(fsm, src):
+  def __repr__(fsm):
+    return str(fsm)
+
+  def __str__(fsm):
+    return 'Fsm(%r,%r)' % (fsm.struct_name, fsm.from_file)
+
+  def parse_states(fsm, src, c_file):
     state = None
     started = None
 
@@ -254,6 +274,15 @@ class Fsm:
     for line in lines:
       state_name = state_starts(line)
       if state_name:
+        state = fsm.find_state_by_name(state_name)
+        if state is not None:
+          if c_file is fsm.from_file:
+            print('ERROR: fsm %r has multiple definitions of state %r' % (fsm, state_name))
+          else:
+            print('ERROR: it appears two FSMs with identical name %r exist in %r and %r'
+                  % (fsm.struct_name, fsm.from_file, c_file))
+          state = None
+          continue
         state = State()
         fsm.states.append(state)
         started = None
@@ -304,8 +333,13 @@ class Fsm:
 
   def ref_out_states(fsm):
     for state in fsm.states:
-      for e in [Edge(fsm.find_state_by_name(n, True)) for n in state.out_state_names]:
-        state.add_out_edge(e)
+      for out_state_name in state.out_state_names:
+        out_state = fsm.find_state_by_name(out_state_name, False)
+        if out_state is None:
+          print('ERROR: fsm %r has a transition to state not part of the FSM: %r'
+                % (fsm, out_state_name))
+          out_state = fsm.have_state(out_state_name, KIND_STATE, color='red')
+        state.add_out_edge(Edge(out_state))
 
   def find_state_by_name(fsm, name, strict=False):
     for state in fsm.states:
@@ -323,7 +357,8 @@ class Fsm:
 
   def add_special_state(fsm, additional_states, name, in_state=None,
                         out_state=None, event_name=None, kind=KIND_FUNC,
-                        state_action=None, label=None, edge_action=None):
+                        state_action=None, label=None, edge_action=None,
+                        style='dotted', arrow_head=None):
     additional_state = None
     for s in additional_states:
       if s.short_name == name:
@@ -349,13 +384,13 @@ class Fsm:
 
     if out_state:
       additional_state.out_state_names.append(out_state.name)
-      additional_state.add_out_edge(Edge(out_state, event_name, 'dotted',
-                                         action=edge_action))
+      additional_state.add_out_edge(Edge(out_state, event_name, style=style,
+                                         action=edge_action, arrow_head=arrow_head))
 
     if in_state:
       in_state.out_state_names.append(additional_state.name)
-      in_state.add_out_edge(Edge(additional_state, event_name, 'dotted',
-                                 action=edge_action))
+      in_state.add_out_edge(Edge(additional_state, event_name, style=style,
+                                 action=edge_action, arrow_head=arrow_head))
 
 
   def find_event_edges(fsm, c_files):
@@ -373,9 +408,17 @@ class Fsm:
       for to_state_name, event_name in transitions:
         if not event_name:
           continue
+        found = False
         for out_edge in state.out_edges:
           if out_edge.to_state.name == to_state_name:
             out_edge.add_event_name(event_name)
+            found = True
+        if not found:
+          sys.stderr.write(
+            "ERROR: %s() triggers a transition to %s, but this is not allowed by the FSM definition\n"
+            % (state.action, to_state_name))
+          state.add_out_edge(Edge(fsm.find_state_by_name(to_state_name, True), event_name,
+                                  color='red'))
 
     additional_states = []
 
@@ -410,7 +453,8 @@ class Fsm:
         # them to the graph as well
         additional_funcs = [f for f in funcs_for_in_event if f not in fsm.action_funcs]
         for af in additional_funcs:
-          fsm.add_special_state(additional_states, af, None, state, in_event_name)
+          fsm.add_special_state(additional_states, af, None, state, in_event_name,
+                                arrow_head='halfopen')
 
     fsm.states.extend(additional_states)
 
@@ -491,13 +535,14 @@ class Fsm:
           meta_calling_fsm.add_out_edge(Edge(meta_called_fsm))
 
 
-  def have_state(fsm, name, kind=KIND_STATE):
+  def have_state(fsm, name, kind=KIND_STATE, color=None):
     state = fsm.find_state_by_name(name)
     if not state:
       state = State()
       state.name = name
       state.short_name = name
       state.kind = kind
+      state.color = color
       fsm.states.append(state)
     return state
 
@@ -505,8 +550,8 @@ class Fsm:
     out = ['digraph G {', 'rankdir=LR;']
 
     for state in fsm.states:
-      out.append('%s [label="%s"%s]' % (state.short_name, state.get_label(),
-                  state.shape_str()))
+      out.append('%s [label="%s"%s%s]' % (state.short_name, state.get_label(),
+                  state.shape_str(), state.color_str()))
 
     for state in fsm.states:
       for out_edge in state.out_edges:
@@ -517,9 +562,16 @@ class Fsm:
         if out_edge.actions:
           labels.extend(out_edge.action_labels())
         if labels:
-          attrs.append('label="%s"' % (r'\n'.join(labels)))
+          label = r'\n'.join(labels)
+        else:
+          label = '-'
+        attrs.append('label="%s"' % label)
         if out_edge.style:
           attrs.append('style=%s'% out_edge.style)
+        if out_edge.color:
+          attrs.append('color=%s'% out_edge.color)
+        if out_edge.arrow_head:
+          attrs.append('arrowhead=%s'% out_edge.arrow_head)
         attrs_str = ''
         if attrs:
           attrs_str = ' [%s]' % (','.join(attrs))
@@ -560,6 +612,9 @@ re_func = re.compile(r'(\b[a-z_][a-z_0-9]*\b)\([^)]*\)\W*^{', re.MULTILINE)
 re_state_trigger = re.compile(r'osmo_fsm_inst_state_chg\([^,]+,\W*([A-Z_][A-Z_0-9]*)\W*,', re.M)
 re_fsm_alloc = re.compile(r'osmo_fsm_inst_alloc[_child]*\(\W*&([a-z_][a-z_0-9]*),', re.M)
 re_fsm_event_dispatch = re.compile(r'osmo_fsm_inst_dispatch\(\W*[^,]+,\W*([A-Z_][A-Z_0-9]*)\W*,', re.M)
+re_comment_multiline = re.compile(r'/\*.*?\*/', re.M | re.S)
+re_comment_single_line = re.compile(r'//.*$', re.M | re.S)
+re_break = re.compile(r'^\W*\bbreak;', re.M)
 
 class CFile():
   def __init__(c_file, path):
@@ -568,6 +623,12 @@ class CFile():
     c_file.funcs = {}
     c_file.fsm_allocators = listdict()
 
+  def __repr__(c_file):
+    return str(c_file)
+
+  def __str__(c_file):
+    return 'CFile(%r)' % c_file.path
+
   def extract_block(c_file, brace_open, brace_close, start):
     pos = 0
     try:
@@ -612,13 +673,15 @@ class CFile():
       states_struct_name = m.group(1)
       for fsm in fsms:
         if states_struct_name == fsm.states_struct_name:
-          fsm.parse_states(c_file.extract_block('{', '}', m.start()))
+          fsm.parse_states(c_file.extract_block('{', '}', m.start()), c_file)
 
   def parse_functions(c_file):
     funcs = {}
     for m in re_func.finditer(c_file.src):
       name = m.group(1)
       func_src = c_file.extract_block('{', '}', m.start())
+      func_src = ''.join(re_comment_multiline.split(func_src))
+      func_src = ''.join(re_comment_single_line.split(func_src))
       funcs[name] = func_src
     c_file.funcs = funcs
     c_file.find_fsm_allocators()
@@ -640,7 +703,9 @@ class CFile():
 
   def find_state_transitions(c_file, event_names):
     TO_STATE = 'TO_STATE'
-    EVENT = 'EVENT'
+    IF_EVENT = 'IF_EVENT'
+    CASE_EVENT = 'CASE_EVENT'
+    BREAK = 'BREAK'
     func_to_state_transitions = listdict()
 
     for func_name, src in c_file.funcs.items():
@@ -651,19 +716,37 @@ class CFile():
         found_tokens.append((m.start(), TO_STATE, to_state))
 
       for event in event_names:
-        re_event = re.compile(r'\b(' + event + r')\b')
+        re_event = re.compile(r'\bif\w*\(.*\b(' + event + r')\b')
         for m in re_event.finditer(src):
           event = m.group(1)
-          found_tokens.append((m.start(), EVENT, event))
+          found_tokens.append((m.start(), IF_EVENT, event))
+
+        re_event = re.compile(r'^\W*case\W\W*\b(' + event + r'):', re.M)
+        for m in re_event.finditer(src):
+          event = m.group(1)
+          found_tokens.append((m.start(), CASE_EVENT, event))
+
+      for m in re_break.finditer(src):
+        found_tokens.append((m.start(), BREAK, 'break'))
 
       found_tokens = sorted(found_tokens)
 
-      last_event = None
+      last_events = []
+      saw_break = True
       for start, kind, name in found_tokens:
-        if kind == EVENT:
-          last_event = name
-        else:
-          func_to_state_transitions.add(func_name, (name, last_event))
+        if kind == IF_EVENT:
+          last_events = [name]
+          saw_break = True
+        elif kind == CASE_EVENT:
+          if saw_break:
+            last_events = []
+            saw_break = False
+          last_events.append(name)
+        elif kind == BREAK:
+          saw_break = True
+        elif kind == TO_STATE:
+          for event in (last_events or [None]):
+            func_to_state_transitions.add(func_name, (name, event))
 
     return func_to_state_transitions
 
@@ -713,6 +796,14 @@ fsms = []
 for c_file in c_files:
   fsms.extend(c_file.find_fsms())
 
+for fsm1 in fsms:
+  for fsm2 in fsms:
+    if fsm1 is fsm2:
+      continue
+    if fsm1.states_struct_name == fsm2.states_struct_name:
+      print('ERROR: two distinct FSMs share the same states-struct name: %r and %r both use %r'
+            % (fsm1, fsm2, fsm1.states_struct_name))
+
 for c_file in c_files:
   c_file.find_fsm_states(fsms)
   c_file.find_event_sources(fsms)
