@@ -44,6 +44,8 @@ union gsm0808_cell_id_u {
 	uint16_t				ci;
 	struct osmo_location_area_id		lai_and_lac;
 	uint16_t				lac;
+	struct osmo_service_area_id		sai;
+	/* osmocom specific: */
 	struct osmo_cell_global_id_ps		global_ps;
 };
 
@@ -125,6 +127,8 @@ uint8_t gsm0808_enc_encrypt_info(struct msgb *msg,
 				 const struct gsm0808_encrypt_info *ei);
 int gsm0808_dec_encrypt_info(struct gsm0808_encrypt_info *ei,
 			     const uint8_t *elem, uint8_t len);
+int gsm0808_enc_kc128(struct msgb *msg, const uint8_t *kc128);
+int gsm0808_dec_kc128(uint8_t *kc128, const uint8_t *elem, uint8_t len);
 uint8_t gsm0808_enc_cell_id_list2(struct msgb *msg, const struct gsm0808_cell_id_list2 *cil);
 uint8_t gsm0808_enc_cell_id_list(struct msgb *msg,
 				 const struct gsm0808_cell_id_list *cil)
