@@ -159,7 +159,7 @@ const struct value_string gprs_ns2_cause_strs[] = {
 	{ 0, NULL }
 };
 
-static const struct rate_ctr_desc nsvc_ctr_description[] = {
+static const struct rate_ctr_desc ns_ctr_description[] = {
 	[NS_CTR_PKTS_IN]	= { "packets:in", 	"Packets at NS Level  ( In)" },
 	[NS_CTR_PKTS_OUT] 	= { "packets:out",	"Packets at NS Level  (Out)" },
 	[NS_CTR_PKTS_OUT_DROP] 	= { "packets:out:drop",	"Dropped Packets      (Out)" },
@@ -177,11 +177,19 @@ static const struct rate_ctr_desc nsvc_ctr_description[] = {
 	[NS_CTR_LOST_RESET]	= { "lost:reset",	"RESET ACK missing count   " },
 };
 
+static const struct rate_ctr_group_desc nse_ctrg_desc = {
+	.group_name_prefix = "ns:nse",
+	.group_description = "NSE Peer Statistics",
+	.num_ctr = ARRAY_SIZE(ns_ctr_description),
+	.ctr_desc = ns_ctr_description,
+	.class_id = OSMO_STATS_CLASS_PEER,
+};
+
 static const struct rate_ctr_group_desc nsvc_ctrg_desc = {
 	.group_name_prefix = "ns:nsvc",
 	.group_description = "NSVC Peer Statistics",
-	.num_ctr = ARRAY_SIZE(nsvc_ctr_description),
-	.ctr_desc = nsvc_ctr_description,
+	.num_ctr = ARRAY_SIZE(ns_ctr_description),
+	.ctr_desc = ns_ctr_description,
 	.class_id = OSMO_STATS_CLASS_PEER,
 };
 
@@ -414,6 +422,8 @@ static struct gprs_ns2_vc *ns2_load_sharing_weight_modulo(
 
 	mod = (bvci + load_selector) % nse->sum_data_weight;
 	llist_for_each_entry(tmp, &nse->nsvc, list) {
+		if (tmp->data_weight == 0)
+			continue;
 		if (!ns2_vc_is_unblocked(tmp))
 			continue;
 		if (i == mod || mod < i + tmp->data_weight)
@@ -560,11 +570,11 @@ void ns2_prim_status_ind(struct gprs_ns2_nse *nse,
 		nsp.u.status.nsvc = gprs_ns2_ll_str_buf(nsvc_str, sizeof(nsvc_str), nsvc);
 		LOGNSVC(nsvc, LOGL_NOTICE, "NS-STATUS.ind(bvci=%05u): cause=%s, transfer=%d, first=%d, mtu=%d\n",
 			nsp.bvci, gprs_ns2_aff_cause_prim_str(nsp.u.status.cause),
-			nsp.u.status.transfer, nsp.u.status.first, nse->mtu);
+			nsp.u.status.transfer, nsp.u.status.first, nsp.u.status.mtu);
 	} else {
 		LOGNSE(nse, LOGL_NOTICE, "NS-STATUS.ind(bvci=%05u): cause=%s, transfer=%d, first=%d, mtu=%d\n",
 			nsp.bvci, gprs_ns2_aff_cause_prim_str(nsp.u.status.cause),
-			nsp.u.status.transfer, nsp.u.status.first, nse->mtu);
+			nsp.u.status.transfer, nsp.u.status.first, nsp.u.status.mtu);
 	}
 
 	osmo_prim_init(&nsp.oph, SAP_NS, GPRS_NS2_PRIM_STATUS, PRIM_OP_INDICATION, NULL);
@@ -606,8 +616,12 @@ struct gprs_ns2_vc *ns2_vc_alloc(struct gprs_ns2_vc_bind *bind, struct gprs_ns2_
 
 	bind->nsi->nsvc_rate_ctr_idx++;
 
-	llist_add(&nsvc->list, &nse->nsvc);
-	llist_add(&nsvc->blist, &bind->nsvc);
+	rate_ctr_group_set_name(nsvc->ctrg, id);
+	osmo_stat_item_group_set_name(nsvc->statg, id);
+
+	llist_add_tail(&nsvc->list, &nse->nsvc);
+	llist_add_tail(&nsvc->blist, &bind->nsvc);
+	osmo_clock_gettime(CLOCK_MONOTONIC, &nsvc->ts_alive_change);
 	ns2_nse_update_mtu(nse);
 
 	return nsvc;
@@ -626,9 +640,9 @@ err:
  *  \param[in] nsvc NS-VC to destroy */
 void gprs_ns2_free_nsvc(struct gprs_ns2_vc *nsvc)
 {
-	if (!nsvc)
+	if (!nsvc || nsvc->freed)
 		return;
-
+	nsvc->freed = true;
 	ns2_prim_status_ind(nsvc->nse, nsvc, 0, GPRS_NS2_AFF_CAUSE_VC_FAILURE);
 
 	llist_del(&nsvc->list);
@@ -651,18 +665,29 @@ void gprs_ns2_free_nsvc(struct gprs_ns2_vc *nsvc)
 	talloc_free(nsvc);
 }
 
+void ns2_free_nsvcs(struct gprs_ns2_nse *nse)
+{
+	struct gprs_ns2_vc *nsvc;
+
+	/* prevent recursive free() when the user reacts on a down event and free() a second time */
+	while (!llist_empty(&nse->nsvc)) {
+		nsvc = llist_first_entry(&nse->nsvc, struct gprs_ns2_vc, list);
+		gprs_ns2_free_nsvc(nsvc);
+	}
+}
+
 /*! Destroy/release all NS-VC of given NSE
  *  \param[in] nse NSE
  */
 void gprs_ns2_free_nsvcs(struct gprs_ns2_nse *nse)
 {
-	struct gprs_ns2_vc *nsvc, *tmp;
-
-	if (!nse)
+	if (!nse || nse->freed)
 		return;
 
-	llist_for_each_entry_safe(nsvc, tmp, &nse->nsvc, list) {
-		gprs_ns2_free_nsvc(nsvc);
+	if (nse->bss_sns_fi) {
+		osmo_fsm_inst_dispatch(nse->bss_sns_fi, NS2_SNS_EV_REQ_FREE_NSVCS, NULL);
+	} else {
+		ns2_free_nsvcs(nse);
 	}
 }
 
@@ -776,12 +801,13 @@ struct gprs_ns2_vc *gprs_ns2_nsvc_by_nsvci(struct gprs_ns2_inst *nsi, uint16_t n
 /*! Create a NS Entity within given NS instance.
  *  \param[in] nsi NS instance in which to create NS Entity
  *  \param[in] nsei NS Entity Identifier of to-be-created NSE
+ *  \param[in] ip_sns_role_sgsn Does local side implement SGSN role?
  *  \returns newly-allocated NS-E in successful case; NULL on error */
-struct gprs_ns2_nse *gprs_ns2_create_nse(struct gprs_ns2_inst *nsi, uint16_t nsei,
-					 enum gprs_ns2_ll linklayer, enum gprs_ns2_dialect dialect)
+struct gprs_ns2_nse *gprs_ns2_create_nse2(struct gprs_ns2_inst *nsi, uint16_t nsei,
+					  enum gprs_ns2_ll linklayer, enum gprs_ns2_dialect dialect,
+					  bool ip_sns_role_sgsn)
 {
 	struct gprs_ns2_nse *nse;
-	char sns[16];
 
 	nse = gprs_ns2_nse_by_nsei(nsi, nsei);
 	if (nse) {
@@ -792,28 +818,77 @@ struct gprs_ns2_nse *gprs_ns2_create_nse(struct gprs_ns2_inst *nsi, uint16_t nse
 	nse = talloc_zero(nsi, struct gprs_ns2_nse);
 	if (!nse)
 		return NULL;
+	nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+	nse->ip_sns_role_sgsn = ip_sns_role_sgsn;
 
-	if (dialect == GPRS_NS2_DIALECT_SNS) {
-		snprintf(sns, sizeof(sns), "NSE%05u-SNS", nsei);
-		nse->bss_sns_fi = ns2_sns_bss_fsm_alloc(nse, sns);
-		if (!nse->bss_sns_fi) {
-			talloc_free(nse);
-			return NULL;
-		}
+	if (ns2_nse_set_dialect(nse, dialect) < 0) {
+		talloc_free(nse);
+		return NULL;
+	}
+
+	nse->ctrg = rate_ctr_group_alloc(nse, &nse_ctrg_desc, nsei);
+	if (!nse->ctrg) {
+		talloc_free(nse);
+		return NULL;
 	}
 
-	nse->dialect = dialect;
 	nse->ll = linklayer;
 	nse->nsei = nsei;
 	nse->nsi = nsi;
 	nse->first = true;
 	nse->mtu = 0;
-	llist_add(&nse->list, &nsi->nse);
+	llist_add_tail(&nse->list, &nsi->nse);
 	INIT_LLIST_HEAD(&nse->nsvc);
+	osmo_clock_gettime(CLOCK_MONOTONIC, &nse->ts_alive_change);
 
 	return nse;
 }
 
+int ns2_nse_set_dialect(struct gprs_ns2_nse *nse, enum gprs_ns2_dialect dialect)
+{
+	char sns[16];
+
+	if (nse->dialect == dialect)
+		return 0;
+
+	switch (nse->dialect) {
+	case GPRS_NS2_DIALECT_UNDEF:
+		if (dialect == GPRS_NS2_DIALECT_SNS) {
+			snprintf(sns, sizeof(sns), "NSE%05u-SNS", nse->nsei);
+			if (nse->ip_sns_role_sgsn)
+				nse->bss_sns_fi = ns2_sns_sgsn_fsm_alloc(nse, sns);
+			else
+				nse->bss_sns_fi = ns2_sns_bss_fsm_alloc(nse, sns);
+			if (!nse->bss_sns_fi)
+				return -1;
+		}
+		nse->dialect = dialect;
+		break;
+	default:
+		if (dialect == GPRS_NS2_DIALECT_UNDEF) {
+			if (nse->bss_sns_fi)
+				osmo_fsm_inst_term(nse->bss_sns_fi, OSMO_FSM_TERM_REQUEST, NULL);
+			nse->bss_sns_fi = NULL;
+			nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+		} else {
+			/* we don't support arbitrary changes without going through UNDEF first */
+			return -EPERM;
+		}
+	}
+
+	return 0;
+}
+
+/*! Create a NS Entity within given NS instance.
+ *  \param[in] nsi NS instance in which to create NS Entity
+ *  \param[in] nsei NS Entity Identifier of to-be-created NSE
+ *  \returns newly-allocated NS-E in successful case; NULL on error */
+struct gprs_ns2_nse *gprs_ns2_create_nse(struct gprs_ns2_inst *nsi, uint16_t nsei,
+					 enum gprs_ns2_ll linklayer, enum gprs_ns2_dialect dialect)
+{
+	return gprs_ns2_create_nse2(nsi, nsei, linklayer, dialect, false);
+}
+
 /*! Return the NSEI
  * \param[in] nse NS Entity
  * \return the nsei.
@@ -827,9 +902,10 @@ uint16_t gprs_ns2_nse_nsei(struct gprs_ns2_nse *nse)
  *  \param[in] nse NS Entity to destroy */
 void gprs_ns2_free_nse(struct gprs_ns2_nse *nse)
 {
-	if (!nse)
+	if (!nse || nse->freed)
 		return;
 
+	nse->freed = true;
 	nse->alive = false;
 	if (nse->bss_sns_fi) {
 		osmo_fsm_inst_term(nse->bss_sns_fi, OSMO_FSM_TERM_REQUEST, NULL);
@@ -838,6 +914,8 @@ void gprs_ns2_free_nse(struct gprs_ns2_nse *nse)
 
 	gprs_ns2_free_nsvcs(nse);
 	ns2_prim_status_ind(nse, NULL, 0, GPRS_NS2_AFF_CAUSE_FAILURE);
+	rate_ctr_group_free(nse->ctrg);
+	ns2_free_nsvcs(nse);
 
 	llist_del(&nse->list);
 	talloc_free(nse);
@@ -845,9 +923,11 @@ void gprs_ns2_free_nse(struct gprs_ns2_nse *nse)
 
 void gprs_ns2_free_nses(struct gprs_ns2_inst *nsi)
 {
-	struct gprs_ns2_nse *nse, *ntmp;
+	struct gprs_ns2_nse *nse;
 
-	llist_for_each_entry_safe(nse, ntmp, &nsi->nse, list) {
+	/* prevent recursive free() when the user reacts on a down event and free() a second time */
+	while (!llist_empty(&nsi->nse)) {
+		nse = llist_first_entry(&nsi->nse, struct gprs_ns2_nse, list);
 		gprs_ns2_free_nse(nse);
 	}
 }
@@ -864,16 +944,67 @@ static inline int ns2_tlv_parse(struct tlv_parsed *dec,
 	return rc;
 }
 
+static enum ns2_cs ns2_create_vc_sns(struct gprs_ns2_vc_bind *bind,
+			  const struct osmo_sockaddr *remote,
+			  struct gprs_ns2_vc **success, uint16_t nsei)
+{
+	struct gprs_ns2_vc *nsvc;
+	struct gprs_ns2_nse *nse;
+
+	nsvc = gprs_ns2_nsvc_by_sockaddr_bind(bind, remote);
+	/* ns2_create_vc() is only called if no NS-VC could be found */
+	OSMO_ASSERT(!nsvc);
+
+	nse = gprs_ns2_nse_by_nsei(bind->nsi, nsei);
+	if (!nse) {
+		if (!bind->accept_sns) {
+			struct osmo_sockaddr_str remote_str;
+			osmo_sockaddr_str_from_sockaddr(&remote_str, &remote->u.sas);
+			/* no dynamic creation of IP-SNS NSE permitted */
+			LOGP(DLNS, LOGL_ERROR, "[%s]:%u: Dynamic creation of NSE(%05u) via IP-SNS not "
+			     "permitted. Check your config.\n", remote_str.ip, remote_str.port, nsei);
+			return NS2_CS_ERROR;
+		}
+		nse = gprs_ns2_create_nse2(bind->nsi, nsei, bind->ll, GPRS_NS2_DIALECT_SNS, true);
+		if (!nse) {
+			LOGP(DLNS, LOGL_ERROR, "Failed to create NSE(%05u)\n", nsei);
+			return NS2_CS_ERROR;
+		}
+		/* add configured list of default binds; if that fails, use only current bind */
+		if (!ns2_sns_add_sns_default_binds(nse))
+			gprs_ns2_sns_add_bind(nse, bind);
+	} else {
+		/* nsei already known */
+		if (nse->ll != bind->ll) {
+			LOGNSE(nse, LOGL_ERROR, "Received NS-RESET with wrong linklayer(%s)"
+				" for already known NSE(%s)\n", gprs_ns2_lltype_str(bind->ll),
+				gprs_ns2_lltype_str(nse->ll));
+			return NS2_CS_SKIPPED;
+		}
+	}
+
+	nsvc = ns2_ip_bind_connect(bind, nse, remote);
+	if (!nsvc)
+		return NS2_CS_SKIPPED;
+
+	nsvc->nsvci_is_valid = false;
+
+	*success = nsvc;
+
+	return NS2_CS_CREATED;
+}
 
 /*! Create a new NS-VC based on a [received] message. Depending on the bind it might create a NSE.
  *  \param[in] bind the bind through which msg was received
  *  \param[in] msg the actual received message
+ *  \param[in] remote address of remote peer sending message
  *  \param[in] logname A name to describe the VC. E.g. ip address pair
  *  \param[out] reject A message filled to be sent back. Only used in failure cases.
  *  \param[out] success A pointer which will be set to the new VC on success
  *  \return enum value indicating the status, e.g. GPRS_NS2_CS_CREATED */
 enum ns2_cs ns2_create_vc(struct gprs_ns2_vc_bind *bind,
 			  struct msgb *msg,
+			  const struct osmo_sockaddr *remote,
 			  const char *logname,
 			  struct msgb **reject,
 			  struct gprs_ns2_vc **success)
@@ -886,7 +1017,8 @@ enum ns2_cs ns2_create_vc(struct gprs_ns2_vc_bind *bind,
 	enum gprs_ns2_vc_mode vc_mode;
 	uint16_t nsvci;
 	uint16_t nsei;
-	char idbuf[32];
+	const struct osmo_sockaddr *local;
+	char idbuf[256], tmp[INET6_ADDRSTRLEN + 8];
 
 	int rc, tlv;
 
@@ -900,6 +1032,20 @@ enum ns2_cs ns2_create_vc(struct gprs_ns2_vc_bind *bind,
 	tlv = ns2_tlv_parse(&tp, nsh->data,
 			   msgb_l2len(msg) - sizeof(*nsh), 0, 0);
 
+	if (bind->ll == GPRS_NS2_LL_UDP && nsh->pdu_type == SNS_PDUT_SIZE && tlv >= 0) {
+		uint16_t nsei;
+
+		if (!TLVP_PRES_LEN(&tp, NS_IE_NSEI, 2)) {
+			rc = reject_status_msg(msg, &tp, reject, NS_CAUSE_MISSING_ESSENT_IE);
+			if (rc < 0)
+				LOGP(DLNS, LOGL_ERROR, "Failed to generate reject message (%d)\n", rc);
+			return NS2_CS_REJECTED;
+		}
+		nsei = tlvp_val16be(&tp, NS_IE_NSEI);
+		/* Create NS-VC, and if required, even NSE dynamically */
+		return ns2_create_vc_sns(bind, remote, success, nsei);
+	}
+
 	switch (nsh->pdu_type) {
 	case NS_PDUT_STATUS:
 		/* Do not respond, see 3GPP TS 08.16, 7.5.1 */
@@ -1005,8 +1151,12 @@ enum ns2_cs ns2_create_vc(struct gprs_ns2_vc_bind *bind,
 
 	nsvci = tlvp_val16be(&tp, NS_IE_VCI);
 	vc_mode = ns2_dialect_to_vc_mode(dialect);
-	snprintf(idbuf, sizeof(idbuf), "%s-NSE%05u-NSVC%05u", gprs_ns2_lltype_str(nse->ll),
-		 nse->nsei, nsvci);
+
+	local = gprs_ns2_ip_bind_sockaddr(bind);
+	osmo_sockaddr_to_str_buf(tmp, sizeof(tmp), local);
+	snprintf(idbuf, sizeof(idbuf), "%s-NSE%05u-NSVC%05u-%s-%s", gprs_ns2_lltype_str(nse->ll),
+		 nse->nsei, nsvci, tmp, osmo_sockaddr_to_str(remote));
+	osmo_identifier_sanitize_buf(idbuf, NULL, '_');
 	nsvc = ns2_vc_alloc(bind, nse, false, vc_mode, idbuf);
 	if (!nsvc)
 		return NS2_CS_SKIPPED;
@@ -1135,7 +1285,7 @@ int gprs_ns2_nse_foreach_nsvc(struct gprs_ns2_nse *nse, gprs_ns2_foreach_nsvc_cb
 
 /*! Bottom-side entry-point for received NS PDU from the driver/bind
  * \param[in] nsvc NS-VC for which the message was received
- * \param msg the received message. Ownership is trasnferred, caller must not free it!
+ * \param msg the received message. Ownership is transferred, caller must not free it!
  * \return 0 on success; negative on error */
 int ns2_recv_vc(struct gprs_ns2_vc *nsvc,
 		struct msgb *msg)
@@ -1147,11 +1297,18 @@ int ns2_recv_vc(struct gprs_ns2_vc *nsvc,
 	log_set_context(LOG_CTX_GB_NSE, nsvc->nse);
 	log_set_context(LOG_CTX_GB_NSVC, nsvc);
 
-	rate_ctr_inc(&nsvc->ctrg->ctr[NS_CTR_PKTS_IN]);
-	rate_ctr_add(&nsvc->ctrg->ctr[NS_CTR_BYTES_IN], msg->len);
+	RATE_CTR_INC_NS(nsvc, NS_CTR_PKTS_IN);
+	RATE_CTR_ADD_NS(nsvc, NS_CTR_BYTES_IN, msg->len);
 
-	if (msg->len < sizeof(struct gprs_ns_hdr))
-		return -EINVAL;
+	if (msg->len < sizeof(struct gprs_ns_hdr)) {
+		rc = -EINVAL;
+		goto freemsg;
+	}
+
+	if (nsh->pdu_type != NS_PDUT_UNITDATA)
+		LOG_NS_RX_SIGNAL(nsvc, nsh->pdu_type);
+	else
+		LOG_NS_DATA(nsvc, "Rx", nsh->pdu_type, LOGL_INFO, "\n");
 
 	switch (nsh->pdu_type) {
 	case SNS_PDUT_CONFIG:
@@ -1160,11 +1317,10 @@ int ns2_recv_vc(struct gprs_ns2_vc *nsvc,
 				   msgb_l2len(msg) - sizeof(*nsh)-1, 0, 0);
 		if (rc < 0) {
 			LOGP(DLNS, LOGL_NOTICE, "Error during TLV Parse in %s\n", msgb_hexdump(msg));
-			return rc;
+			goto freemsg;
 		}
 		/* All sub-network service related message types */
-		rc = ns2_sns_rx(nsvc, msg, &tp);
-		break;
+		return ns2_sns_rx(nsvc, msg, &tp);
 	case SNS_PDUT_ACK:
 	case SNS_PDUT_ADD:
 	case SNS_PDUT_CHANGE_WEIGHT:
@@ -1174,14 +1330,13 @@ int ns2_recv_vc(struct gprs_ns2_vc *nsvc,
 				   msgb_l2len(msg) - sizeof(*nsh)-5, 0, 0);
 		if (rc < 0) {
 			LOGP(DLNS, LOGL_NOTICE, "Error during TLV Parse in %s\n", msgb_hexdump(msg));
-			return rc;
+			goto freemsg;
 		}
 		tp.lv[NS_IE_NSEI].val = nsh->data+2;
 		tp.lv[NS_IE_NSEI].len = 2;
 		tp.lv[NS_IE_TRANS_ID].val = nsh->data+4;
 		tp.lv[NS_IE_TRANS_ID].len = 1;
-		rc = ns2_sns_rx(nsvc, msg, &tp);
-		break;
+		return ns2_sns_rx(nsvc, msg, &tp);
 	case SNS_PDUT_CONFIG_ACK:
 	case SNS_PDUT_SIZE:
 	case SNS_PDUT_SIZE_ACK:
@@ -1189,27 +1344,25 @@ int ns2_recv_vc(struct gprs_ns2_vc *nsvc,
 				   msgb_l2len(msg) - sizeof(*nsh), 0, 0);
 		if (rc < 0) {
 			LOGP(DLNS, LOGL_NOTICE, "Error during TLV Parse in %s\n", msgb_hexdump(msg));
-			return rc;
+			goto freemsg;
 		}
 		/* All sub-network service related message types */
-		rc = ns2_sns_rx(nsvc, msg, &tp);
-		break;
-
+		return ns2_sns_rx(nsvc, msg, &tp);
 	case NS_PDUT_UNITDATA:
-		rc = ns2_vc_rx(nsvc, msg, &tp);
-		break;
+		return ns2_vc_rx(nsvc, msg, &tp);
 	default:
 		rc = ns2_tlv_parse(&tp, nsh->data,
 				   msgb_l2len(msg) - sizeof(*nsh), 0, 0);
 		if (rc < 0) {
 			LOGP(DLNS, LOGL_NOTICE, "Error during TLV Parse\n");
 			if (nsh->pdu_type != NS_PDUT_STATUS)
-				ns2_tx_status(nsvc, NS_CAUSE_PROTO_ERR_UNSPEC, 0, msg);
+				ns2_tx_status(nsvc, NS_CAUSE_PROTO_ERR_UNSPEC, 0, msg, NULL);
 			return rc;
 		}
-		rc = ns2_vc_rx(nsvc, msg, &tp);
-		break;
+		return ns2_vc_rx(nsvc, msg, &tp);
 	}
+freemsg:
+	msgb_free(msg);
 
 	return rc;
 }
@@ -1239,16 +1392,22 @@ void ns2_nse_data_sum(struct gprs_ns2_nse *nse)
 void ns2_nse_notify_unblocked(struct gprs_ns2_vc *nsvc, bool unblocked)
 {
 	struct gprs_ns2_nse *nse = nsvc->nse;
+	struct gprs_ns2_inst *nsi = nse->nsi;
+	uint16_t nsei = nse->nsei;
 
-	ns2_sns_notify_alive(nse, nsvc, unblocked);
 	ns2_nse_data_sum(nse);
+	ns2_sns_notify_alive(nse, nsvc, unblocked);
 
-	if (unblocked == nse->alive)
+	/* NSE could have been freed, try to get it again */
+	nse = gprs_ns2_nse_by_nsei(nsi, nsei);
+
+	if (!nse || unblocked == nse->alive)
 		return;
 
 	/* wait until both data_weight and sig_weight are != 0 before declaring NSE as alive */
 	if (unblocked && nse->sum_data_weight && nse->sum_sig_weight) {
 		nse->alive = true;
+		osmo_clock_gettime(CLOCK_MONOTONIC, &nse->ts_alive_change);
 		ns2_prim_status_ind(nse, NULL, 0, GPRS_NS2_AFF_CAUSE_RECOVERY);
 		nse->first = false;
 		return;
@@ -1257,6 +1416,7 @@ void ns2_nse_notify_unblocked(struct gprs_ns2_vc *nsvc, bool unblocked)
 	if (nse->alive && (nse->sum_data_weight == 0 || nse->sum_sig_weight == 0)) {
 		/* nse became unavailable */
 		nse->alive = false;
+		osmo_clock_gettime(CLOCK_MONOTONIC, &nse->ts_alive_change);
 		ns2_prim_status_ind(nse, NULL, 0, GPRS_NS2_AFF_CAUSE_FAILURE);
 	}
 }
@@ -1289,6 +1449,7 @@ struct gprs_ns2_inst *gprs_ns2_instantiate(void *ctx, osmo_prim_cb cb, void *cb_
 	nsi->timeout[NS_TOUT_TSNS_PROV] = 3; /* 1..10 */
 	nsi->timeout[NS_TOUT_TSNS_SIZE_RETRIES] = 3;
 	nsi->timeout[NS_TOUT_TSNS_CONFIG_RETRIES] = 3;
+	nsi->timeout[NS_TOUT_TSNS_PROCEDURES_RETRIES] = 3;
 
 	return nsi;
 }
@@ -1314,6 +1475,9 @@ void gprs_ns2_start_alive_all_nsvcs(struct gprs_ns2_nse *nse)
 	OSMO_ASSERT(nse);
 
 	llist_for_each_entry(nsvc, &nse->nsvc, list) {
+		/* A pre-configured endpoint shall not be used for NSE data or signalling traffic
+		 * (with the exception of Size and Configuration procedures) unless it is
+		 * configured by the SGSN using the auto-configuration procedures */
 		if (nsvc->sns_only)
 			continue;
 
@@ -1325,14 +1489,11 @@ void gprs_ns2_start_alive_all_nsvcs(struct gprs_ns2_nse *nse)
  *  \param[in] bind the bind we want to destroy */
 void gprs_ns2_free_bind(struct gprs_ns2_vc_bind *bind)
 {
-	struct gprs_ns2_vc *nsvc, *tmp;
+	struct gprs_ns2_vc *nsvc;
 	struct gprs_ns2_nse *nse;
-	if (!bind)
+	if (!bind || bind->freed)
 		return;
-
-	llist_for_each_entry_safe(nsvc, tmp, &bind->nsvc, blist) {
-		gprs_ns2_free_nsvc(nsvc);
-	}
+	bind->freed = true;
 
 	if (gprs_ns2_is_ip_bind(bind)) {
 		llist_for_each_entry(nse, &bind->nsi->nse, list) {
@@ -1340,6 +1501,12 @@ void gprs_ns2_free_bind(struct gprs_ns2_vc_bind *bind)
 		}
 	}
 
+	/* prevent recursive free() when the user reacts on a down event and free() a second time */
+	while (!llist_empty(&bind->nsvc)) {
+		nsvc = llist_first_entry(&bind->nsvc, struct gprs_ns2_vc, blist);
+		gprs_ns2_free_nsvc(nsvc);
+	}
+
 	if (bind->driver->free_bind)
 		bind->driver->free_bind(bind);
 
@@ -1351,9 +1518,11 @@ void gprs_ns2_free_bind(struct gprs_ns2_vc_bind *bind)
 
 void gprs_ns2_free_binds(struct gprs_ns2_inst *nsi)
 {
-	struct gprs_ns2_vc_bind *bind, *tbind;
+	struct gprs_ns2_vc_bind *bind;
 
-	llist_for_each_entry_safe(bind, tbind, &nsi->binding, list) {
+	/* prevent recursive free() when the user reacts on a down event and free() a second time */
+	while (!llist_empty(&nsi->binding)) {
+		bind = llist_first_entry(&nsi->binding, struct gprs_ns2_vc_bind, list);
 		gprs_ns2_free_bind(bind);
 	}
 }
@@ -1428,7 +1597,7 @@ void ns2_nse_update_mtu(struct gprs_ns2_nse *nse)
 
 	nse->mtu = mtu;
 	if (nse->alive)
-		ns2_prim_status_ind(nsvc->nse, NULL, 0, GPRS_NS2_AFF_CAUSE_MTU_CHANGE);
+		ns2_prim_status_ind(nse, NULL, 0, GPRS_NS2_AFF_CAUSE_MTU_CHANGE);
 }
 
 /*! calculate the transfer capabilities for a nse
@@ -1455,8 +1624,9 @@ int ns2_count_transfer_cap(struct gprs_ns2_nse *nse,
 		if (ns2_vc_is_unblocked(nsvc))
 			active_nsvcs++;
 	}
-	/* an alive nse should always have active_nsvcs */
-	OSMO_ASSERT(active_nsvcs);
+
+	if (!active_nsvcs)
+		return 0;
 
 	active_binds = talloc_zero_array(nse, struct gprs_ns2_vc_bind*, active_nsvcs);
 	if (!active_binds)
@@ -1510,7 +1680,7 @@ int ns2_bind_alloc(struct gprs_ns2_inst *nsi, const char *name,
 	bind->sns_data_weight = 1;
 	bind->nsi = nsi;
 	INIT_LLIST_HEAD(&bind->nsvc);
-	llist_add(&bind->list, &nsi->binding);
+	llist_add_tail(&bind->list, &nsi->binding);
 
 	nsi->bind_rate_ctr_idx++;
 
