@@ -15,10 +15,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include <string.h>
@@ -228,22 +224,32 @@ struct msgb *gsm0808_create_clear_command2(uint8_t cause, bool csfb_ind)
 	return msg;
 }
 
-/*! Create BSSMAP Cipher Mode Command message
+/*! Superseded by gsm0808_create_cipher2() to include Kc128.
+ * Create BSSMAP Cipher Mode Command message (without Kc128).
  *  \param[in] ei Mandatory Encryption Information
+ *  \param[in] kc128 optional kc128 key for A5/4
  *  \param[in] cipher_response_mode optional 1-byte Cipher Response Mode
  *  \returns callee-allocated msgb with BSSMAP Cipher Mode Command message */
 struct msgb *gsm0808_create_cipher(const struct gsm0808_encrypt_info *ei,
 				   const uint8_t *cipher_response_mode)
+{
+	struct gsm0808_cipher_mode_command cmc = {
+		.ei = *ei,
+		.cipher_response_mode_present = (cipher_response_mode != NULL),
+		.cipher_response_mode = (cipher_response_mode ? *cipher_response_mode : 0),
+	};
+	return gsm0808_create_cipher2(&cmc);
+}
+
+/*! Create BSSMAP Cipher Mode Command message.
+ *  \param[in] cmc  Information to encode.
+ *  \returns callee-allocated msgb with BSSMAP Cipher Mode Command message */
+struct msgb *gsm0808_create_cipher2(const struct gsm0808_cipher_mode_command *cmc)
 {
 	/* See also: 3GPP TS 48.008 3.2.1.30 CIPHER MODE COMMAND */
 	struct msgb *msg;
 
-	/* Mandatory emelent! */
-	OSMO_ASSERT(ei);
-
-	msg =
-	    msgb_alloc_headroom(BSSMAP_MSG_SIZE, BSSMAP_MSG_HEADROOM,
-				"cipher-mode-command");
+	msg = msgb_alloc_headroom(BSSMAP_MSG_SIZE, BSSMAP_MSG_HEADROOM, "cipher-mode-command");
 	if (!msg)
 		return NULL;
 
@@ -251,12 +257,16 @@ struct msgb *gsm0808_create_cipher(const struct gsm0808_encrypt_info *ei,
 	msgb_v_put(msg, BSS_MAP_MSG_CIPHER_MODE_CMD);
 
 	/* Encryption Information 3.2.2.10 */
-	gsm0808_enc_encrypt_info(msg, ei);
+	gsm0808_enc_encrypt_info(msg, &cmc->ei);
 
 	/* Cipher Response Mode 3.2.2.34 */
-	if (cipher_response_mode)
+	if (cmc->cipher_response_mode_present)
 		msgb_tv_put(msg, GSM0808_IE_CIPHER_RESPONSE_MODE,
-			    *cipher_response_mode);
+			    cmc->cipher_response_mode);
+
+	/* Kc128 3.2.2.109 */
+	if (cmc->kc128_present)
+		gsm0808_enc_kc128(msg, cmc->kc128);
 
 	/* pre-pend the header */
 	msg->l3h =
@@ -791,7 +801,7 @@ struct msgb *gsm0808_create_paging(const char *imsi, const uint32_t *tmsi,
 {
 	struct gsm0808_cell_id_list2 cil2 = {};
 
-	/* Mandatory emelents! */
+	/* Mandatory elements! */
 	OSMO_ASSERT(cil);
 
 	if (cil->id_list_len > GSM0808_CELL_ID_LIST2_MAXLEN)
@@ -833,6 +843,11 @@ static uint8_t put_old_bss_to_new_bss_information(struct msgb *msg,
 		msgb_tlv_put(msg, GSM0808_FE_IE_CURRENT_CHANNEL_TYPE_2, 2, val);
 	}
 
+	if (i->last_eutran_plmn_id_present) {
+		msgb_put_u8(msg, GSM0808_FE_IE_LAST_USED_EUTRAN_PLMN_ID);
+		osmo_plmn_to_bcd(msgb_put(msg, 3), &i->last_eutran_plmn_id);
+	}
+
 	*tlv_len = (uint8_t) (msg->tail - old_tail);
 	return *tlv_len + 2;
 }
@@ -986,6 +1001,9 @@ struct msgb *gsm0808_create_handover_request(const struct gsm0808_handover_reque
 		msgb_tv_fixed_put(msg, GSM0808_IE_CALL_ID, 4, val);
 	}
 
+	if (params->more_items && params->kc128_present)
+		gsm0808_enc_kc128(msg, params->kc128);
+
 	if (params->global_call_reference && params->global_call_reference_len) {
 		msgb_tlv_put(msg, GSM0808_IE_GLOBAL_CALL_REF,
 			     params->global_call_reference_len, params->global_call_reference);
@@ -1027,6 +1045,12 @@ struct msgb *gsm0808_create_handover_request_ack2(const struct gsm0808_handover_
 	if (params->aoip_transport_layer)
 		gsm0808_enc_aoip_trasp_addr(msg, params->aoip_transport_layer);
 
+	/* AoIP: add Codec List (BSS Supported) 3.2.2.103.
+	 * (codec_list_bss_supported was added to struct gsm0808_handover_request_ack later than speech_codec_chosen
+	 * below, but it needs to come before it in the message coding). */
+	if (params->more_items && params->codec_list_bss_supported.len)
+		gsm0808_enc_speech_codec_list(msg, &params->codec_list_bss_supported);
+
 	/* AoIP: Speech Codec (Chosen) 3.2.2.104 */
 	if (params->speech_codec_chosen_present)
 		gsm0808_enc_speech_codec(msg, &params->speech_codec_chosen);
@@ -1501,6 +1525,7 @@ static const struct tlv_definition bss_att_tlvdef = {
 		[GSM0808_IE_LOCATION_ESTIMATE]		= { TLV_TYPE_TLV },
 		[GSM0808_IE_POSITIONING_DATA]		= { TLV_TYPE_TLV },
 		[GSM0808_IE_LCS_CAUSE]			= { TLV_TYPE_TLV },
+		[GSM0808_IE_LCS_CLIENT_TYPE]		= { TLV_TYPE_TLV },
 		[GSM0808_IE_APDU]			= { TLV_TYPE_TLV },
 		[GSM0808_IE_NETWORK_ELEMENT_IDENTITY]	= { TLV_TYPE_TLV },
 		[GSM0808_IE_GPS_ASSISTANCE_DATA]	= { TLV_TYPE_TLV },
@@ -1517,6 +1542,7 @@ static const struct tlv_definition bss_att_tlvdef = {
 		[GSM0800_IE_INTER_SYSTEM_INFO]		= { TLV_TYPE_TLV },
 		[GSM0808_IE_SNA_ACCESS_INFO]		= { TLV_TYPE_TLV },
 		[GSM0808_IE_VSTK_RAND_INFO]		= { TLV_TYPE_TLV },
+		[GSM0808_IE_VSTK_INFO]			= { TLV_TYPE_TLV },
 		[GSM0808_IE_PAGING_INFO]		= { TLV_TYPE_TV },
 		[GSM0808_IE_IMEI]			= { TLV_TYPE_TLV },
 		[GSM0808_IE_VELOCITY_ESTIMATE]		= { TLV_TYPE_TLV },
@@ -1559,6 +1585,11 @@ static const struct tlv_definition bss_att_tlvdef = {
 		[GSM0808_IE_CN_TO_MS_TRANSP_INFO]	= { TLV_TYPE_TLV },
 		[GSM0808_IE_SELECTED_PLMN_ID]		= { TLV_TYPE_FIXED, 3 },
 		[GSM0808_IE_LAST_USED_EUTRAN_PLMN_ID]	= { TLV_TYPE_FIXED, 3 },
+		[GSM0808_IE_OLD_LAI]			= { TLV_TYPE_FIXED, 5 },
+		[GSM0808_IE_ATTACH_INDICATOR]		= { TLV_TYPE_T },
+		[GSM0808_IE_SELECTED_OPERATOR]		= { TLV_TYPE_FIXED, 3 },
+		[GSM0808_IE_PS_REGISTERED_OPERATOR]	= { TLV_TYPE_FIXED, 3 },
+		[GSM0808_IE_CS_REGISTERED_OPERATOR]	= { TLV_TYPE_FIXED, 3 },
 
 		/* Osmocom extensions */
 		[GSM0808_IE_OSMO_OSMUX_SUPPORT]		= { TLV_TYPE_T },
@@ -1571,6 +1602,32 @@ const struct tlv_definition *gsm0808_att_tlvdef(void)
 	return &bss_att_tlvdef;
 }
 
+/* As per 3GPP TS 48.008 version 16.0.0 Release 16 § 3.2.2.58 Old BSS to New BSS Information  */
+const struct tlv_definition gsm0808_old_bss_to_new_bss_info_att_tlvdef = {
+	.def = {
+		[GSM0808_FE_IE_EXTRA_INFORMATION] 			= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_CURRENT_CHANNEL_TYPE_2]			= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_TARGET_CELL_RADIO_INFORMATION] 		= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_GPRS_SUSPEND_INFORMATION] 		= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_MULTIRATE_CONFIGURATION_INFORMATION] 	= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_DUAL_TRANSFER_MODE_INFORMATION] 		= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_INTER_RAT_HANDOVER_INFO] 		= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_CDMA2000_CAPABILITY_INFORMATION] 	= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_DOWNLINK_CELL_LOAD_INFORMATION] 		= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_UPLINK_CELL_LOAD_INFORMATION] 		= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_CELL_LOAD_INFORMATION_GROUP] 		= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_CELL_LOAD_INFORMATION] 			= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_PS_INDICATION] 				= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_DTM_HANDOVER_COMMAND_INDICATION] 	= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_D_RNTI] 					= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_IRAT_MEASUREMENT_CONFIGURATION] 		= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_SOURCE_CELL_ID] 				= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_IRAT_MEASUREMENT_CONFIGURATION_EXTENDED_E_ARFCNS] = { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_VGCS_TALKER_MODE] 			= { TLV_TYPE_TLV },
+		[GSM0808_FE_IE_LAST_USED_EUTRAN_PLMN_ID] 		= { TLV_TYPE_FIXED, 3 },
+	},
+};
+
 const struct value_string gsm0406_dlci_sapi_names[] = {
 	{ DLCI_SAPI_RR_MM_CC,	"RR/MM/CC" },
 	{ DLCI_SAPI_SMS,	"SMS" },
