@@ -19,10 +19,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include <stdint.h>
@@ -124,6 +120,7 @@ const struct tlv_definition gsm48_rr_att_tlvdef = {
 		[GSM48_IE_CHDES_2_AFTER]	= { TLV_TYPE_FIXED, 3 },
 		[GSM48_IE_MODE_SEC_CH]		= { TLV_TYPE_TV },
 		[GSM48_IE_F_CH_SEQ_AFTER]		= { TLV_TYPE_FIXED, 9 },
+		[GSM48_IE_EXTENDED_TSC_SET]	= { TLV_TYPE_TV },
 		[GSM48_IE_MA_AFTER]		= { TLV_TYPE_TLV },
 		[GSM48_IE_BA_RANGE]		= { TLV_TYPE_TLV },
 		[GSM48_IE_GROUP_CHDES]		= { TLV_TYPE_TLV },
@@ -164,13 +161,16 @@ static const struct value_string rr_cause_names[] = {
 	{ GSM48_RR_CAUSE_ABNORMAL_TIMER,	"Abnormal release, timer expired" },
 	{ GSM48_RR_CAUSE_ABNORMAL_NOACT,	"Abnormal release, no activity on radio path" },
 	{ GSM48_RR_CAUSE_PREMPTIVE_REL,		"Preemptive release" },
+	{ GSM48_RR_CAUSE_UTRAN_CFG_UNK,		"UTRAN configuration unknown" },
 	{ GSM48_RR_CAUSE_HNDOVER_IMP,		"Handover impossible, timing advance out of range" },
 	{ GSM48_RR_CAUSE_CHAN_MODE_UNACCT,	"Channel mode unacceptable" },
 	{ GSM48_RR_CAUSE_FREQ_NOT_IMPL,		"Frequency not implemented" },
+	{ GSM48_RR_CAUSE_LEAVE_GROUP_CA,	"Originator or talker leaving group call area" },
+	{ GSM48_RR_CAUSE_LOW_LEVEL_FAIL,	"Lower layer failure" },
 	{ GSM48_RR_CAUSE_CALL_CLEARED,		"Call already cleared" },
 	{ GSM48_RR_CAUSE_SEMANT_INCORR,		"Semantically incorrect message" },
 	{ GSM48_RR_CAUSE_INVALID_MAND_INF,	"Invalid mandatory information" },
-	{ GSM48_RR_CAUSE_MSG_TYPE_N,		"Message type non-existant or not implemented" },
+	{ GSM48_RR_CAUSE_MSG_TYPE_N,		"Message type non-existent or not implemented" },
 	{ GSM48_RR_CAUSE_MSG_TYPE_N_COMPAT,	"Message type not compatible with protocol state" },
 	{ GSM48_RR_CAUSE_COND_IE_ERROR,		"Conditional IE error" },
 	{ GSM48_RR_CAUSE_NO_CELL_ALLOC_A,	"No cell allocation available" },
@@ -429,9 +429,52 @@ const struct value_string gsm48_chan_mode_names[] = {
 	{ GSM48_CMODE_DATA_12k0,	"DATA_12k0" },
 	{ GSM48_CMODE_DATA_6k0,		"DATA_6k0" },
 	{ GSM48_CMODE_DATA_3k6,		"DATA_3k6" },
+	{ GSM48_CMODE_SPEECH_V1_VAMOS,	"SPEECH_V1_VAMOS" },
+	{ GSM48_CMODE_SPEECH_V2_VAMOS,	"SPEECH_V2_VAMOS" },
+	{ GSM48_CMODE_SPEECH_V3_VAMOS,	"SPEECH_V3_VAMOS" },
+	{ GSM48_CMODE_SPEECH_V5_VAMOS,	"SPEECH_V5_VAMOS" },
 	{ 0,				NULL },
 };
 
+/*! Translate GSM48_CMODE_SPEECH_* to its corresponding GSM48_CMODE_SPEECH_*_VAMOS mode.
+ * If the mode has no equivalent VAMOS mode, return a negative value.
+ */
+enum gsm48_chan_mode gsm48_chan_mode_to_vamos(enum gsm48_chan_mode mode)
+{
+	switch (mode) {
+	case GSM48_CMODE_SPEECH_V1:
+	case GSM48_CMODE_SPEECH_V1_VAMOS:
+		return GSM48_CMODE_SPEECH_V1_VAMOS;
+	case GSM48_CMODE_SPEECH_EFR:
+	case GSM48_CMODE_SPEECH_V2_VAMOS:
+		return GSM48_CMODE_SPEECH_V2_VAMOS;
+	case GSM48_CMODE_SPEECH_AMR:
+	case GSM48_CMODE_SPEECH_V3_VAMOS:
+		return GSM48_CMODE_SPEECH_V3_VAMOS;
+	case GSM48_CMODE_SPEECH_V5_VAMOS:
+		return GSM48_CMODE_SPEECH_V5_VAMOS;
+	default:
+		return -1;
+	}
+}
+
+/*! Translate GSM48_CMODE_SPEECH_*_VAMOS to its corresponding GSM48_CMODE_SPEECH_* non-vamos mode.
+ * If the mode is not a VAMOS mode, return the unchanged mode.
+ */
+enum gsm48_chan_mode gsm48_chan_mode_to_non_vamos(enum gsm48_chan_mode mode)
+{
+	switch (mode) {
+	case GSM48_CMODE_SPEECH_V1_VAMOS:
+		return GSM48_CMODE_SPEECH_V1;
+	case GSM48_CMODE_SPEECH_V2_VAMOS:
+		return GSM48_CMODE_SPEECH_EFR;
+	case GSM48_CMODE_SPEECH_V3_VAMOS:
+		return GSM48_CMODE_SPEECH_AMR;
+	default:
+		return mode;
+	}
+}
+
 const struct value_string gsm_chan_t_names[] = {
 	{ GSM_LCHAN_NONE,	"NONE" },
 	{ GSM_LCHAN_SDCCH,	"SDCCH" },
@@ -554,8 +597,8 @@ int osmo_mobile_identity_decode(struct osmo_mobile_identity *mi, const uint8_t *
 {
 	int rc;
 	int nibbles_len;
-	char *str;
-	size_t str_size;
+	char *str = NULL; /* initialize to avoid uninitialized false warnings on some gcc versions (11.1.0) */
+	size_t str_size = 0; /* initialize to avoid uninitialized false warnings on some gcc versions (11.1.0) */
 
 	if (!mi_data || mi_len < 1)
 		return -EBADMSG;
@@ -630,8 +673,12 @@ int osmo_mobile_identity_decode(struct osmo_mobile_identity *mi, const uint8_t *
 			goto return_error;
 		}
 		rc = osmo_bcd2str(str, str_size, mi_data, 1, 1 + nibbles_len, allow_hex);
-		/* rc checked below */
-		break;
+		/* check mi->str printing rc */
+		if (rc < 1 || rc >= str_size) {
+			rc = -EBADMSG;
+			goto return_error;
+		}
+		return 0;
 
 	default:
 		/* Already handled above, but as future bug paranoia: */
@@ -639,13 +686,6 @@ int osmo_mobile_identity_decode(struct osmo_mobile_identity *mi, const uint8_t *
 		goto return_error;
 	}
 
-	/* check mi->str printing rc */
-	if (rc < 1 || rc >= str_size) {
-		rc = -EBADMSG;
-		goto return_error;
-	}
-	return 0;
-
 return_error:
 	*mi = (struct osmo_mobile_identity){
 		.type = GSM_MI_TYPE_NONE,
@@ -1295,6 +1335,25 @@ int gsm48_construct_ra(uint8_t *buf, const struct gprs_ra_id *raid)
 	return 6;
 }
 
+/*! Compare a TS 04.08 Routing Area Identifier
+ *  \param[in] raid1 first Routing Area ID to compare.
+ *  \param[in] raid2 second Routing Area ID to compare.
+ *  \returns true if raid1 and raid2 match, false otherwise. */
+bool gsm48_ra_equal(const struct gprs_ra_id *raid1, const struct gprs_ra_id *raid2)
+{
+	if (raid1->mcc != raid2->mcc)
+		return false;
+	if (raid1->mnc != raid2->mnc)
+		return false;
+	if (raid1->mnc_3_digits != raid2->mnc_3_digits)
+		return false;
+	if (raid1->lac != raid2->lac)
+		return false;
+	if (raid1->rac != raid2->rac)
+		return false;
+	return true;
+}
+
 /*! Determine number of paging sub-channels
  *  \param[in] chan_desc Control Channel Description
  *  \returns number of paging sub-channels
