@@ -46,8 +46,9 @@ int osmo_gsm48_rest_octets_si1_encode(uint8_t *data, uint8_t *nch_pos, int is180
 	if (nch_pos) {
 		bitvec_set_bit(&bv, H);
 		bitvec_set_uint(&bv, *nch_pos, 5);
-	} else
+	} else {
 		bitvec_set_bit(&bv, L);
+	}
 
 	if (is1800_net)
 		bitvec_set_bit(&bv, L);
@@ -63,6 +64,7 @@ static inline bool append_eutran_neib_cell(struct bitvec *bv, const struct osmo_
 					   uint8_t budget)
 {
 	unsigned i, skip = 0;
+	size_t offset = *e_offset;
 	int16_t rem = budget - 6; /* account for mandatory stop bit and THRESH_E-UTRAN_high */
 	uint8_t earfcn_budget;
 
@@ -93,7 +95,7 @@ static inline bool append_eutran_neib_cell(struct bitvec *bv, const struct osmo_
 	/* now we can proceed with actually adding EARFCNs within adjusted budget limit */
 	for (i = 0; i < e->length; i++) {
 		if (e->arfcn[i] != OSMO_EARFCN_INVALID) {
-			if (skip < *e_offset) {
+			if (skip < offset) {
 				skip++; /* ignore EARFCNs added on previous calls */
 			} else {
 				earfcn_budget = 17; /* compute budget per-EARFCN */
@@ -134,8 +136,9 @@ static inline bool append_eutran_neib_cell(struct bitvec *bv, const struct osmo_
 		/* E-UTRAN_PRIORITY: 3GPP TS 45.008*/
 		bitvec_set_bit(bv, 1);
 		bitvec_set_uint(bv, e->prio, 3);
-	} else
+	} else {
 		bitvec_set_bit(bv, 0);
+	}
 
 	/* THRESH_E-UTRAN_high */
 	bitvec_set_uint(bv, e->thresh_hi, 5);
@@ -144,15 +147,17 @@ static inline bool append_eutran_neib_cell(struct bitvec *bv, const struct osmo_
 		/* THRESH_E-UTRAN_low: */
 		bitvec_set_bit(bv, 1);
 		bitvec_set_uint(bv, e->thresh_lo, 5);
-	} else
+	} else {
 		bitvec_set_bit(bv, 0);
+	}
 
 	if (e->qrxlm_valid) {
 		/* E-UTRAN_QRXLEVMIN: */
 		bitvec_set_bit(bv, 1);
 		bitvec_set_uint(bv, e->qrxlm, 5);
-	} else
+	} else {
 		bitvec_set_bit(bv, 0);
+	}
 
 	return true;
 }
@@ -161,7 +166,7 @@ static inline void append_earfcn(struct bitvec *bv, const struct osmo_earfcn_si2
 {
 	bool appended;
 	unsigned int old = bv->cur_bit; /* save current position to make rollback possible */
-	int rem = budget - 25;
+	int rem = ((int)budget) - 40;
 	if (rem <= 0)
 		return;
 
@@ -189,8 +194,29 @@ static inline void append_earfcn(struct bitvec *bv, const struct osmo_earfcn_si2
 	/* Priority and E-UTRAN Parameters Description */
 	bitvec_set_bit(bv, 1);
 
-	/* No Serving Cell Priority Parameters Descr. */
-	bitvec_set_bit(bv, 0);
+	/* budget: 10 bits used above */
+
+	/* Serving Cell Priority Parameters Descr. is Present,
+	* see also: 3GPP TS 44.018, Table 10.5.2.33b.1 */
+	bitvec_set_bit(bv, 1);
+
+	/* GERAN_PRIORITY */
+	bitvec_set_uint(bv, 0, 3);
+
+	/* THRESH_Priority_Search */
+	bitvec_set_uint(bv, 0, 4);
+
+	/* THRESH_GSM_low */
+	bitvec_set_uint(bv, 0, 4);
+
+	/* H_PRIO */
+	bitvec_set_uint(bv, 0, 2);
+
+	/* T_Reselection */
+	bitvec_set_uint(bv, 0, 2);
+
+	/* budget: 26 bits used above */
+
 	/* No 3G Priority Parameters Description */
 	bitvec_set_bit(bv, 0);
 	/* E-UTRAN Parameters Description */
@@ -214,12 +240,16 @@ static inline void append_earfcn(struct bitvec *bv, const struct osmo_earfcn_si2
 	/* Repeated E-UTRAN Neighbour Cells */
 	bitvec_set_bit(bv, 1);
 
+	/* budget: 34 bits used above */
+
 	appended = append_eutran_neib_cell(bv, e, e_offset, rem);
 	if (!appended) { /* appending is impossible within current budget: rollback */
 		bv->cur_bit = old;
 		return;
 	}
 
+	/* budget: further 6 bits used below, totalling 40 bits */
+
 	/* stop bit - end of Repeated E-UTRAN Neighbour Cells sequence: */
 	bitvec_set_bit(bv, 0);
 
@@ -404,8 +434,9 @@ static inline void append_uarfcns(struct bitvec *bv, const uint16_t *uarfcn_list
 			if (i < uarfcn_length) {
 				cu = uarfcn_list[i];
 				st = i;
-			} else
+			} else {
 				break;
+			}
 		}
 
 	/* stop bit - end of Repeated UTRAN FDD Neighbour Cells */
@@ -508,8 +539,9 @@ static void append_selection_params(struct bitvec *bv,
 		bitvec_set_uint(bv, sp->cell_resel_off, 6);
 		bitvec_set_uint(bv, sp->temp_offs, 3);
 		bitvec_set_uint(bv, sp->penalty_time, 5);
-	} else
+	} else {
 		bitvec_set_bit(bv, L);
+	}
 }
 
 /* Append power offset to bitvec */
@@ -519,8 +551,9 @@ static void append_power_offset(struct bitvec *bv,
 	if (po->present) {
 		bitvec_set_bit(bv, H);
 		bitvec_set_uint(bv, po->power_offset, 2);
-	} else
+	} else {
 		bitvec_set_bit(bv, L);
+	}
 }
 
 /* Append GPRS indicator to bitvec */
@@ -532,8 +565,9 @@ static void append_gprs_ind(struct bitvec *bv,
 		bitvec_set_uint(bv, gi->ra_colour, 3);
 		/* 0 == SI13 in BCCH Norm, 1 == SI13 sent on BCCH Ext */
 		bitvec_set_bit(bv, gi->si13_position);
-	} else
+	} else {
 		bitvec_set_bit(bv, L);
+	}
 }
 
 /* Generate SI3 Rest Octests (Chapter 10.5.2.34 / Table 10.4.72) */
@@ -567,8 +601,9 @@ int osmo_gsm48_rest_octets_si3_encode(uint8_t *data, const struct osmo_gsm48_si_
 	if (si3->scheduling.present) {
 		bitvec_set_bit(&bv, H);
 		bitvec_set_uint(&bv, si3->scheduling.where, 3);
-	} else
+	} else {
 		bitvec_set_bit(&bv, L);
+	}
 
 	/* GPRS Indicator */
 	append_gprs_ind(&bv, &si3->gprs_ind);
@@ -576,9 +611,9 @@ int osmo_gsm48_rest_octets_si3_encode(uint8_t *data, const struct osmo_gsm48_si_
 	/* 3G Early Classmark Sending Restriction. If H, then controlled by
 	 * early_cm_ctrl above */
 	if (si3->early_cm_restrict_3g)
-		bitvec_set_bit(&bv, H);
-	else
 		bitvec_set_bit(&bv, L);
+	else
+		bitvec_set_bit(&bv, H);
 
 	if (si3->si2quater_indicator) {
 		bitvec_set_bit(&bv, H); /* indicator struct present */
@@ -618,22 +653,25 @@ int osmo_gsm48_rest_octets_si4_encode(uint8_t *data, const struct osmo_gsm48_si_
 		if (si4->lsa_params.present) {
 			bitvec_set_bit(&bv, H);
 			append_lsa_params(&bv, &si4->lsa_params);
-		} else
+		} else {
 			bitvec_set_bit(&bv, L);
+		}
 
 		/* Cell Identity */
 		if (1) {
 			bitvec_set_bit(&bv, H);
 			bitvec_set_uint(&bv, si4->cell_id, 16);
-		} else
+		} else {
 			bitvec_set_bit(&bv, L);
+		}
 
 		/* LSA ID Information */
 		if (0) {
 			bitvec_set_bit(&bv, H);
 			/* FIXME */
-		} else
+		} else {
 			bitvec_set_bit(&bv, L);
+		}
 	} else {
 		/* L and break indicator */
 		bitvec_set_bit(&bv, L);
@@ -671,25 +709,29 @@ int osmo_gsm48_rest_octets_si6_encode(uint8_t *data, const struct osmo_gsm48_si6
 		if (in->pch_nch_info.call_priority_present) {
 			bitvec_set_bit(&bv, 1);
 			bitvec_set_uint(&bv, in->pch_nch_info.call_priority, 3);
-		} else
+		} else {
 			bitvec_set_bit(&bv, 0);
+		}
 		bitvec_set_bit(&bv, !!in->pch_nch_info.nln_status_sacch);
-	} else
+	} else {
 		bitvec_set_bit(&bv, L);
+	}
 
 	if (in->vbs_vgcs_options.present) {
 		bitvec_set_bit(&bv, H);
 		bitvec_set_bit(&bv, !!in->vbs_vgcs_options.inband_notifications);
 		bitvec_set_bit(&bv, !!in->vbs_vgcs_options.inband_pagings);
-	} else
+	} else {
 		bitvec_set_bit(&bv, L);
+	}
 
 	if (in->dtm_support.present) {
 		bitvec_set_bit(&bv, H);
 		bitvec_set_uint(&bv, in->dtm_support.rac, 8);
 		bitvec_set_uint(&bv, in->dtm_support.max_lapdm, 3);
-	} else
+	} else {
 		bitvec_set_bit(&bv, L);
+	}
 
 	if (in->band_indicator_1900)
 		bitvec_set_bit(&bv, H);
@@ -699,13 +741,127 @@ int osmo_gsm48_rest_octets_si6_encode(uint8_t *data, const struct osmo_gsm48_si6
 	if (in->gprs_ms_txpwr_max_ccch.present) {
 		bitvec_set_bit(&bv, H);
 		bitvec_set_uint(&bv, in->gprs_ms_txpwr_max_ccch.max_txpwr, 5);
-	} else
+	} else {
 		bitvec_set_bit(&bv, L);
+	}
 
 	bitvec_spare_padding(&bv, (bv.data_len * 8) - 1);
 	return bv.data_len;
 }
 
+
+static unsigned int decode_t3192(unsigned int t3192)
+{
+	/* See also 3GPP TS 44.060
+	   Table 12.24.2: GPRS Cell Options information element details */
+	static const unsigned int decode_t3192_tbl[8] = {500, 1000, 1500, 0, 80, 120, 160, 200};
+	OSMO_ASSERT(t3192 <= 7);
+	return decode_t3192_tbl[t3192];
+}
+
+static unsigned int decode_drx_timer(unsigned int drx)
+{
+	static const unsigned int decode_drx_timer_tbl[8] = {0, 1, 2, 4, 8, 16, 32, 64};
+	OSMO_ASSERT(drx <= 7);
+	return decode_drx_timer_tbl[drx];
+}
+
+static int decode_gprs_cell_opt(struct osmo_gprs_cell_options *gco, struct bitvec *bv)
+{
+	gco->nmo = bitvec_get_uint(bv, 2);
+	gco->t3168 = (bitvec_get_uint(bv, 3) + 1) * 500;
+	gco->t3192 = decode_t3192(bitvec_get_uint(bv, 3));
+	gco->drx_timer_max = decode_drx_timer(bitvec_get_uint(bv, 3));
+
+	/* ACCESS_BURST_TYPE: */
+	bitvec_get_uint(bv, 1);
+	/* CONTROL_ACK_TYPE: */
+	gco->ctrl_ack_type_use_block = bitvec_get_uint(bv, 1);
+	gco->bs_cv_max = bitvec_get_uint(bv, 4);
+
+	if (bitvec_get_uint(bv, 1)) {
+		bitvec_get_uint(bv, 3);	/* DEC */
+		bitvec_get_uint(bv, 3);	/* INC */
+		bitvec_get_uint(bv, 3);	/* MAX */
+	}
+
+	if (bitvec_get_uint(bv, 1)) {
+		int ext_len = bitvec_get_uint(bv, 6);
+		if (ext_len < 0)
+			return ext_len;
+		unsigned int cur_bit = bv->cur_bit;
+		/* Extension Information */
+		/* R99 extension: */
+		gco->ext_info.egprs_supported = bitvec_get_uint(bv, 1);
+		if (gco->ext_info.egprs_supported) {
+			gco->ext_info.use_egprs_p_ch_req = !bitvec_get_uint(bv, 1);
+			gco->ext_info.bep_period = bitvec_get_uint(bv, 4);
+		}
+		gco->ext_info.pfc_supported = bitvec_get_uint(bv, 1);
+		gco->ext_info.dtm_supported = bitvec_get_uint(bv, 1);
+		gco->ext_info.bss_paging_coordination = bitvec_get_uint(bv, 1);
+		/* REL-4 extension: */
+		gco->ext_info.ccn_active = bitvec_get_uint(bv, 1);
+		bitvec_get_uint(bv, 1); /* NW_EXT_UTBF */
+		bv->cur_bit = cur_bit + ext_len + 1;
+	}
+	return 0;
+}
+
+static void decode_gprs_pwr_ctrl_pars(struct osmo_gprs_power_ctrl_pars *pcp, struct bitvec *bv)
+{
+	pcp->alpha = bitvec_get_uint(bv, 4);
+	pcp->t_avg_w = bitvec_get_uint(bv,5);
+	pcp->t_avg_t = bitvec_get_uint(bv, 5);
+	pcp->pc_meas_chan = bitvec_get_uint(bv, 1);
+	pcp->n_avg_i = bitvec_get_uint(bv, 4);
+}
+
+/*! Decode SI13 Rest Octests (04.08 Chapter 10.5.2.37b).
+ *  \param[out] si13 decoded SI13 rest octets
+ *  \param[in] encoded SI13 rest octets
+ *  \returns parsed bits on success, negative on error */
+int osmo_gsm48_rest_octets_si13_decode(struct osmo_gsm48_si13_info *si13, const uint8_t *data)
+{
+	struct osmo_gprs_cell_options *co = &si13->cell_opts;
+	struct osmo_gprs_power_ctrl_pars *pcp = &si13->pwr_ctrl_pars;
+	struct bitvec bv;
+	int rc;
+
+	memset(&bv, 0, sizeof(bv));
+	bv.data = (uint8_t *) data;
+	bv.data_len = 20;
+
+	memset(si13, 0, sizeof(*si13));
+
+
+	if (bitvec_get_bit_high(&bv) == H) {
+		si13->bcch_change_mark = bitvec_get_uint(&bv, 3);
+		si13->si_change_field = bitvec_get_uint(&bv, 4);
+		if (bitvec_get_uint(&bv, 1)) {
+			si13->bcch_change_mark = bitvec_get_uint(&bv, 2);
+			/* FIXME: implement parsing GPRS Mobile Allocation IE */
+			return -ENOTSUP;
+		}
+		if (bitvec_get_uint(&bv, 1)) {
+			/* PBCCH present in cell */
+			/* FIXME: parse not implemented */
+			return -ENOTSUP;
+		} else {
+			/* PBCCH not present in cell */
+			si13->rac = bitvec_get_uint(&bv, 8);
+			si13->spgc_ccch_sup = bitvec_get_uint(&bv, 1);
+			si13->prio_acc_thr = bitvec_get_uint(&bv, 3);
+			si13->net_ctrl_ord = bitvec_get_uint(&bv, 2);
+			if ((rc = decode_gprs_cell_opt(co, &bv)) < 0)
+				return rc;
+
+			decode_gprs_pwr_ctrl_pars(pcp, &bv);
+		}
+	}
+	return bv.cur_bit;
+}
+
 /* GPRS Mobile Allocation as per TS 04.60 Chapter 12.10a:
    < GPRS Mobile Allocation IE > ::=
      < HSN : bit (6) >
@@ -730,8 +886,9 @@ static int append_gprs_mobile_alloc(struct bitvec *bv)
 		/* We want to use a RFL number list */
 		bitvec_set_bit(bv, 1);
 		/* FIXME: RFL number list */
-	} else
+	} else {
 		bitvec_set_bit(bv, 0);
+	}
 
 	if (0) {
 		/* We want to use a MA_BITMAP */
@@ -743,8 +900,9 @@ static int append_gprs_mobile_alloc(struct bitvec *bv)
 			/* We want to provide an ARFCN index list */
 			bitvec_set_bit(bv, 1);
 			/* FIXME */
-		} else
+		} else {
 			bitvec_set_bit(bv, 0);
+		}
 	}
 	return 0;
 }
@@ -862,24 +1020,20 @@ static int append_gprs_cell_opt(struct bitvec *bv,
 	} else {
 		/* extension information */
 		bitvec_set_bit(bv, 1);
+		/* R99 extension: */
 		if (!gco->ext_info.egprs_supported) {
 			/* 6bit length of extension */
-			bitvec_set_uint(bv, (1 + 3)-1, 6);
+			bitvec_set_uint(bv, (1 + 5)-1, 6);
 			/* EGPRS supported in the cell */
 			bitvec_set_bit(bv, 0);
 		} else {
 			/* 6bit length of extension */
-			bitvec_set_uint(bv, (1 + 5 + 3)-1, 6);
+			bitvec_set_uint(bv, (1 + 5 + 5)-1, 6);
 			/* EGPRS supported in the cell */
 			bitvec_set_bit(bv, 1);
 
-			/* 1bit EGPRS PACKET CHANNEL REQUEST */
-			if (gco->supports_egprs_11bit_rach == 0) {
-				bitvec_set_bit(bv,
-					gco->ext_info.use_egprs_p_ch_req);
-			} else {
-				bitvec_set_bit(bv, 0);
-			}
+			/* 1bit EGPRS PACKET CHANNEL REQUEST (inverted logic) */
+			bitvec_set_bit(bv, !gco->ext_info.use_egprs_p_ch_req);
 
 			/* 4bit BEP PERIOD */
 			bitvec_set_uint(bv, gco->ext_info.bep_period, 4);
@@ -887,6 +1041,10 @@ static int append_gprs_cell_opt(struct bitvec *bv,
 		bitvec_set_bit(bv, gco->ext_info.pfc_supported);
 		bitvec_set_bit(bv, gco->ext_info.dtm_supported);
 		bitvec_set_bit(bv, gco->ext_info.bss_paging_coordination);
+
+		/* REL-4 extension: */
+		bitvec_set_bit(bv, gco->ext_info.ccn_active);
+		bitvec_set_bit(bv, 0); /* NW_EXT_UTBF disabled */
 	}
 
 	return 0;
@@ -973,15 +1131,17 @@ void osmo_gsm48_rest_octets_si3_decode(struct osmo_gsm48_si_ro_info *si3, const
 		sp->cell_resel_off = bitvec_get_uint(&bv, 6);
 		sp->temp_offs = bitvec_get_uint(&bv, 3);
 		sp->penalty_time = bitvec_get_uint(&bv, 5);
-	} else
+	} else {
 		sp->present = 0;
+	}
 
 	/* Optional Power Offset */
 	if (bitvec_get_bit_high(&bv) == H) {
 		po->present = 1;
 		po->power_offset = bitvec_get_uint(&bv, 2);
-	} else
+	} else {
 		po->present = 0;
+	}
 
 	/* System Information 2ter Indicator */
 	if (bitvec_get_bit_high(&bv) == H)
@@ -999,26 +1159,71 @@ void osmo_gsm48_rest_octets_si3_decode(struct osmo_gsm48_si_ro_info *si3, const
 	if (bitvec_get_bit_high(&bv) == H) {
 		si3->scheduling.present = 1;
 		si3->scheduling.where = bitvec_get_uint(&bv, 3);
-	} else
+	} else {
 		si3->scheduling.present = 0;
+	}
 
 	/* GPRS Indicator */
 	if (bitvec_get_bit_high(&bv) == H) {
 		gi->present = 1;
 		gi->ra_colour = bitvec_get_uint(&bv, 3);
 		gi->si13_position = bitvec_get_uint(&bv, 1);
-	} else
+	} else {
 		gi->present = 0;
+	}
 
 	/* 3G Early Classmark Sending Restriction. If H, then controlled by
 	 * early_cm_ctrl above */
 	if (bitvec_get_bit_high(&bv) == H)
-		si3->early_cm_restrict_3g = 1;
-	else
 		si3->early_cm_restrict_3g = 0;
+	else
+		si3->early_cm_restrict_3g = 1;
 
 	if (bitvec_get_bit_high(&bv) == H)
 		si3->si2quater_indicator = 1;
 	else
 		si3->si2quater_indicator = 0;
 }
+
+
+void osmo_gsm48_rest_octets_si4_decode(struct osmo_gsm48_si_ro_info *si4, const uint8_t *data, int len)
+{
+	struct osmo_gsm48_si_selection_params *sp = &si4->selection_params;
+	struct osmo_gsm48_si_power_offset *po = &si4->power_offset;
+	struct osmo_gsm48_si3_gprs_ind *gi = &si4->gprs_ind;
+	struct bitvec bv;
+
+	memset(&bv, 0, sizeof(bv));
+	bv.data = (uint8_t *) data;
+	bv.data_len = len;
+
+	memset(si4, 0, sizeof(*si4));
+
+	/* Optional Selection Parameters */
+	if (bitvec_get_bit_high(&bv) == H) {
+		sp->present = 1;
+		sp->cbq = bitvec_get_uint(&bv, 1);
+		sp->cell_resel_off = bitvec_get_uint(&bv, 6);
+		sp->temp_offs = bitvec_get_uint(&bv, 3);
+		sp->penalty_time = bitvec_get_uint(&bv, 5);
+	} else {
+		sp->present = 0;
+	}
+
+	/* Optional Power Offset */
+	if (bitvec_get_bit_high(&bv) == H) {
+		po->present = 1;
+		po->power_offset = bitvec_get_uint(&bv, 2);
+	} else {
+		po->present = 0;
+	}
+
+	/* GPRS Indicator */
+	if (bitvec_get_bit_high(&bv) == H) {
+		gi->present = 1;
+		gi->ra_colour = bitvec_get_uint(&bv, 3);
+		gi->si13_position = bitvec_get_uint(&bv, 1);
+	} else {
+		gi->present = 0;
+	}
+}
