@@ -19,10 +19,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 /*! \mainpage libosmogsm Documentation
@@ -329,12 +325,13 @@ int gsm_septet_pack(uint8_t *result, const uint8_t *rdata, size_t septet_len, ui
 	int i = 0, z = 0;
 	uint8_t cb, nb;
 	int shift = 0;
-	uint8_t *data = calloc(septet_len + 1, sizeof(uint8_t));
+	uint8_t *data = malloc(septet_len + 1);
 
 	if (padding) {
 		shift = 7 - padding;
 		/* the first zero is needed for padding */
 		memcpy(data + 1, rdata, septet_len);
+		data[0] = 0x00;
 		septet_len++;
 	} else
 		memcpy(data, rdata, septet_len);
@@ -388,7 +385,7 @@ int gsm_7bit_encode_n(uint8_t *result, size_t n, const char *data, int *octets)
 	size_t max_septets = n * 8 / 7;
 
 	/* prepare for the worst case, every character expanding to two bytes */
-	uint8_t *rdata = calloc(strlen(data) * 2, sizeof(uint8_t));
+	uint8_t *rdata = malloc(strlen(data) * 2);
 	y = gsm_septet_encode(rdata, data);
 
 	if (y > max_septets) {
@@ -493,7 +490,7 @@ int osmo_get_rand_id(uint8_t *out, size_t len)
  *  \param[out] buf Pre-allocated bufer for storing IE
  *  \returns Number of bytes filled in buf
  */
-size_t gsm0858_rsl_ul_meas_enc(struct gsm_meas_rep_unidir *mru, bool dtxd_used,
+size_t gsm0858_rsl_ul_meas_enc(const struct gsm_meas_rep_unidir *mru, bool dtxd_used,
 			uint8_t *buf)
 {
 	buf[0] = dtxd_used ? (1 << 6) : 0;
