@@ -18,10 +18,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 /*! \addtogroup lapd
@@ -946,7 +942,7 @@ static int lapd_rx_u_sabm(struct msgb *msg, struct lapd_msg_ctx *lctx)
 		dl->cont_res = lapd_msgb_alloc(length, "CONT RES");
 		memcpy(msgb_put(dl->cont_res, length), msg->l3h,
 			length);
-		LOGDL(dl, LOGL_NOTICE, "Store content res.\n");
+		LOGDL(dl, LOGL_INFO, "Store content res.\n");
 	}
 	/* send notification to L3 */
 	if (length == 0) {
