@@ -19,10 +19,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 /*! \addtogroup lapdm
@@ -336,8 +332,8 @@ static void lapdm_pad_msgb(struct msgb *msg, uint8_t n201)
 		return;
 	}
 
-	data = msgb_put(msg, pad_len);
-	memset(data, 0x2B, pad_len);
+	data = msgb_put(msg, pad_len); /* TODO: random padding */
+	memset(data, GSM_MACBLOCK_PADDING, pad_len);
 }
 
 /* input function that L2 calls when sending messages up to L3 */
@@ -735,11 +731,9 @@ static int l2_ph_data_ind(struct msgb *msg, struct lapdm_entity *le,
 			 && LAPDm_CTRL_U_BITS(msg->l2h[3]) == 0) {
 				mctx.lapdm_fmt = LAPDm_FMT_B4;
 				n201 = N201_B4;
-				LOGP(DLLAPD, LOGL_INFO, "fmt=B4\n");
 			} else {
 				mctx.lapdm_fmt = LAPDm_FMT_B;
 				n201 = N201_AB_SACCH;
-				LOGP(DLLAPD, LOGL_INFO, "fmt=B\n");
 			}
 			/* SACCH frames have a two-byte L1 header that
 			 * OsmocomBB L1 doesn't strip */
@@ -750,7 +744,6 @@ static int l2_ph_data_ind(struct msgb *msg, struct lapdm_entity *le,
 			sapi = (msg->l2h[0] >> 2) & 7;
 		} else {
 			mctx.lapdm_fmt = LAPDm_FMT_B;
-			LOGP(DLLAPD, LOGL_INFO, "fmt=B\n");
 			n201 = N201_AB_SDCCH;
 			sapi = (msg->l2h[0] >> 2) & 7;
 		}
@@ -850,7 +843,6 @@ static int l2_ph_data_ind(struct msgb *msg, struct lapdm_entity *le,
 		break;
 	case LAPDm_FMT_Bbis:
 		/* directly pass up to layer3 */
-		LOGP(DLLAPD, LOGL_INFO, "fmt=Bbis UI\n");
 		msg->l3h = msg->l2h;
 		msgb_pull_to_l3(msg);
 		rc = send_rslms_rll_l3(RSL_MT_UNIT_DATA_IND, &mctx, msg);
