@@ -21,10 +21,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 /*! \addtogroup logging
@@ -38,6 +34,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdbool.h>
+#include <unistd.h>
 
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
@@ -50,9 +47,12 @@
 #include <osmocom/core/logging.h>
 #include <osmocom/core/timer.h>
 #include <osmocom/core/byteswap.h>
+#include <osmocom/core/thread.h>
 
 #define	GSMTAP_LOG_MAX_SIZE 4096
 
+static __thread uint32_t logging_gsmtap_tid;
+
 static void _gsmtap_raw_output(struct log_target *target, int subsys,
 			       unsigned int level, const char *file,
 			       int line, int cont, const char *format,
@@ -82,6 +82,9 @@ static void _gsmtap_raw_output(struct log_target *target, int subsys,
 	/* Logging header */
 	golh = (struct gsmtap_osmocore_log_hdr *) msgb_put(msg, sizeof(*golh));
 	OSMO_STRLCPY_ARRAY(golh->proc_name, target->tgt_gsmtap.ident);
+	if (logging_gsmtap_tid == 0)
+		osmo_store32be((uint32_t)osmo_gettid(), &logging_gsmtap_tid);
+	golh->pid = logging_gsmtap_tid;
 	if (subsys_name)
 		OSMO_STRLCPY_ARRAY(golh->subsys, subsys_name + 1);
 	else
@@ -102,6 +105,12 @@ static void _gsmtap_raw_output(struct log_target *target, int subsys,
 	if (rc < 0) {
 		msgb_free(msg);
 		return;
+	} else if (rc >= msgb_tailroom(msg)) {
+		/* If the output was truncated, vsnprintf() returns the
+		 * number of characters which would have been written
+		 * if enough space had been available (excluding '\0'). */
+		rc = msgb_tailroom(msg);
+		msg->tail[rc - 1]  = '\0';
 	}
 	msgb_put(msg, rc);
 
@@ -111,7 +120,11 @@ static void _gsmtap_raw_output(struct log_target *target, int subsys,
 }
 
 /*! Create a new logging target for GSMTAP logging
+ *  \param[in] host remote host to send the logs to
+ *  \param[in] port remote port to send the logs to
  *  \param[in] ident string identifier
+ *  \param[in] ofd_wq_mode register osmo_wqueue (1) or not (0)
+ *  \param[in] add_sink add GSMTAP sink or not
  *  \returns Log target in case of success, NULL in case of error
  */
 struct log_target *log_target_create_gsmtap(const char *host, uint16_t port,
