@@ -15,10 +15,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include "../config.h"
@@ -118,7 +114,7 @@ static struct addrinfo *addrinfo_helper(uint16_t family, uint16_t type, uint8_t
 static int addrinfo_helper_multi(struct addrinfo **addrinfo, uint16_t family, uint16_t type, uint8_t proto,
 					const char **hosts, size_t host_cnt, uint16_t port, bool passive)
 {
-	int i, j;
+	unsigned int i, j;
 
 	for (i = 0; i < host_cnt; i++) {
 		addrinfo[i] = addrinfo_helper(family, type, proto, hosts[i], port, passive);
@@ -214,7 +210,8 @@ static int socket_helper_osa(const struct osmo_sockaddr *addr, uint16_t type, ui
 static int multiaddr_snprintf(char* buf, size_t buf_len, const char **hosts, size_t host_cnt)
 {
 	int len = 0, offset = 0, rem = buf_len;
-	int ret, i;
+	size_t i;
+	int ret;
 	char *after;
 
 	if (buf_len < 3)
@@ -487,8 +484,8 @@ int osmo_sock_init2(uint16_t family, uint16_t type, uint8_t proto,
 }
 
 #define _SOCKADDR_TO_STR(dest, sockaddr) do { \
-		if (osmo_sockaddr_str_from_sockaddr(&dest, &sockaddr->u.sas)) \
-			osmo_strlcpy(dest.ip, "Invalid IP", 11); \
+		if (osmo_sockaddr_str_from_sockaddr(dest, &sockaddr->u.sas)) \
+			osmo_strlcpy((dest)->ip, "Invalid IP", 11); \
 	} while (0)
 
 /*! Initialize a socket (including bind and/or connect)
@@ -523,7 +520,8 @@ int osmo_sock_init_osa(uint16_t type, uint8_t proto,
 		        unsigned int flags)
 {
 	int sfd = -1, rc, on = 1;
-	struct osmo_sockaddr_str sastr = {};
+	struct osmo_sockaddr_str _sastr = {};
+	struct osmo_sockaddr_str *sastr = &_sastr;
 
 	if ((flags & (OSMO_SOCK_F_BIND | OSMO_SOCK_F_CONNECT)) == 0) {
 		LOGP(DLGLOBAL, LOGL_ERROR, "invalid: you have to specify either "
@@ -554,8 +552,8 @@ int osmo_sock_init_osa(uint16_t type, uint8_t proto,
 		sfd = socket_helper_osa(local, type, proto, flags);
 		if (sfd < 0) {
 			_SOCKADDR_TO_STR(sastr, local);
-			LOGP(DLGLOBAL, LOGL_ERROR, "no suitable local addr found for: %s:%u\n",
-				sastr.ip, sastr.port);
+			LOGP(DLGLOBAL, LOGL_ERROR, "no suitable local addr found for: " OSMO_SOCKADDR_STR_FMT "\n",
+			     OSMO_SOCKADDR_STR_FMT_ARGS(sastr));
 			return -ENODEV;
 		}
 
@@ -565,10 +563,8 @@ int osmo_sock_init_osa(uint16_t type, uint8_t proto,
 			if (rc < 0) {
 				_SOCKADDR_TO_STR(sastr, local);
 				LOGP(DLGLOBAL, LOGL_ERROR,
-				     "cannot setsockopt socket:"
-				     " %s:%u: %s\n",
-				     sastr.ip, sastr.port,
-				     strerror(errno));
+				     "cannot setsockopt socket: " OSMO_SOCKADDR_STR_FMT ": %s\n",
+				     OSMO_SOCKADDR_STR_FMT_ARGS(sastr), strerror(errno));
 				close(sfd);
 				return rc;
 			}
@@ -576,8 +572,8 @@ int osmo_sock_init_osa(uint16_t type, uint8_t proto,
 
 		if (bind(sfd, &local->u.sa, sizeof(struct osmo_sockaddr)) == -1) {
 			_SOCKADDR_TO_STR(sastr, local);
-			LOGP(DLGLOBAL, LOGL_ERROR, "unable to bind socket: %s:%u: %s\n",
-			     sastr.ip, sastr.port, strerror(errno));
+			LOGP(DLGLOBAL, LOGL_ERROR, "unable to bind socket: " OSMO_SOCKADDR_STR_FMT ": %s\n",
+				     OSMO_SOCKADDR_STR_FMT_ARGS(sastr), strerror(errno));
 			close(sfd);
 			return -1;
 		}
@@ -599,8 +595,8 @@ int osmo_sock_init_osa(uint16_t type, uint8_t proto,
 		rc = connect(sfd, &remote->u.sa, sizeof(struct osmo_sockaddr));
 		if (rc != 0 && errno != EINPROGRESS) {
 			_SOCKADDR_TO_STR(sastr, remote);
-			LOGP(DLGLOBAL, LOGL_ERROR, "unable to connect socket: %s:%u: %s\n",
-			     sastr.ip, sastr.port, strerror(errno));
+			LOGP(DLGLOBAL, LOGL_ERROR, "unable to connect socket: " OSMO_SOCKADDR_STR_FMT ": %s\n",
+			     OSMO_SOCKADDR_STR_FMT_ARGS(sastr), strerror(errno));
 			close(sfd);
 			return rc;
 		}
@@ -669,7 +665,7 @@ static int socket_helper_multiaddr(uint16_t family, uint16_t type, uint8_t proto
 /* Build array of addresses taking first addrinfo result of the requested family
  * for each host in addrs_buf. */
 static int addrinfo_to_sockaddr(uint16_t family, const struct addrinfo **result,
-				const char **hosts, int host_cont,
+				const char **hosts, unsigned int host_cont,
 				uint8_t *addrs_buf, size_t addrs_buf_len) {
 	size_t host_idx, offset = 0;
 	const struct addrinfo *rp;
@@ -725,7 +721,7 @@ int osmo_sock_init2_multiaddr(uint16_t family, uint16_t type, uint8_t proto,
 {
 	struct addrinfo *res_loc[OSMO_SOCK_MAX_ADDRS], *res_rem[OSMO_SOCK_MAX_ADDRS];
 	int sfd = -1, rc, on = 1;
-	int i;
+	unsigned int i;
 	bool loc_has_v4addr, rem_has_v4addr;
 	bool loc_has_v6addr, rem_has_v6addr;
 	struct sockaddr_in6 addrs_buf[OSMO_SOCK_MAX_ADDRS];
@@ -1258,6 +1254,23 @@ uint16_t osmo_sockaddr_port(const struct sockaddr *sa)
 	return 0;
 }
 
+/*! Set sockaddr port content (to network byte order).
+ *  \param[out] sa  sockaddr to set the port of.
+ *  \param[in] port  port nr to set.
+ */
+void osmo_sockaddr_set_port(struct sockaddr *sa, uint16_t port)
+{
+	struct osmo_sockaddr *osa = (struct osmo_sockaddr *)sa;
+	switch (osa->u.sa.sa_family) {
+	case AF_INET6:
+		osa->u.sin6.sin6_port = htons(port);
+		return;
+	case AF_INET:
+		osa->u.sin.sin_port = htons(port);
+		return;
+	}
+}
+
 /*! Initialize a unix domain socket (including bind/connect)
  *  \param[in] type Socket type like SOCK_DGRAM, SOCK_STREAM
  *  \param[in] proto Protocol like IPPROTO_TCP, IPPROTO_UDP
@@ -1737,6 +1750,65 @@ int osmo_sockaddr_local_ip(struct osmo_sockaddr *local_ip, const struct osmo_soc
 	return rc;
 }
 
+/*! Copy the addr part, the IP address octets in network byte order, to a buffer.
+ * Useful for encoding network protocols.
+ * \param[out] dst  Write octets to this buffer.
+ * \param[in] dst_maxlen  Space available in buffer.
+ * \param[in] os  Sockaddr to copy IP of.
+ * \return nr of octets written on success, negative on error.
+ */
+int osmo_sockaddr_to_octets(uint8_t *dst, size_t dst_maxlen, const struct osmo_sockaddr *os)
+{
+	const void *addr;
+	size_t len;
+	switch (os->u.sa.sa_family) {
+	case AF_INET:
+		addr = &os->u.sin.sin_addr;
+		len = sizeof(os->u.sin.sin_addr);
+		break;
+	case AF_INET6:
+		addr = &os->u.sin6.sin6_addr;
+		len = sizeof(os->u.sin6.sin6_addr);
+		break;
+	default:
+		return -ENOTSUP;
+	}
+	if (dst_maxlen < len)
+		return -ENOSPC;
+	memcpy(dst, addr, len);
+	return len;
+}
+
+/*! Copy the addr part, the IP address octets in network byte order, from a buffer.
+ * Useful for decoding network protocols.
+ * \param[out] os  Write IP address to this sockaddr.
+ * \param[in] src  Source buffer to read IP address octets from.
+ * \param[in] src_len  Number of octets to copy.
+ * \return number of octets read on success, negative on error.
+ */
+int osmo_sockaddr_from_octets(struct osmo_sockaddr *os, const void *src, size_t src_len)
+{
+	void *addr;
+	size_t len;
+	*os = (struct osmo_sockaddr){0};
+	switch (src_len) {
+	case sizeof(os->u.sin.sin_addr):
+		os->u.sa.sa_family = AF_INET;
+		addr = &os->u.sin.sin_addr;
+		len = sizeof(os->u.sin.sin_addr);
+		break;
+	case sizeof(os->u.sin6.sin6_addr):
+		os->u.sin6.sin6_family = AF_INET6;
+		addr = &os->u.sin6.sin6_addr;
+		len = sizeof(os->u.sin6.sin6_addr);
+		break;
+	default:
+		return -ENOTSUP;
+	}
+	memcpy(addr, src, len);
+	return len;
+}
+
 /*! Compare two osmo_sockaddr.
  * \param[in] a
  * \param[in] b
@@ -1779,7 +1851,57 @@ const char *osmo_sockaddr_to_str(const struct osmo_sockaddr *sockaddr)
 	return osmo_sockaddr_to_str_buf(buf, sizeof(buf), sockaddr);
 }
 
-/*! string-format a given osmo_sockaddr address into a user-supplied buffer
+/*! string-format a given osmo_sockaddr address into a user-supplied buffer.
+ * Same as osmo_sockaddr_to_str_buf() but returns a would-be length in snprintf() style.
+ *  \param[in] buf user-supplied output buffer
+ *  \param[in] buf_len size of the user-supplied output buffer in bytes
+ *  \param[in] sockaddr the osmo_sockaddr to print
+ *  \return number of characters that would be written if the buffer is large enough, like snprintf().
+ */
+int osmo_sockaddr_to_str_buf2(char *buf, size_t buf_len, const struct osmo_sockaddr *sockaddr)
+{
+	struct osmo_strbuf sb = { .buf = buf, .len = buf_len };
+	uint16_t port = 0;
+
+	if (!sockaddr) {
+		OSMO_STRBUF_PRINTF(sb, "NULL");
+		return sb.chars_needed;
+	}
+
+	switch (sockaddr->u.sa.sa_family) {
+	case AF_INET:
+		OSMO_STRBUF_APPEND(sb, osmo_sockaddr_to_str_and_uint, &port, &sockaddr->u.sa);
+		if (port)
+			OSMO_STRBUF_PRINTF(sb, ":%u", port);
+		break;
+	case AF_INET6:
+		OSMO_STRBUF_PRINTF(sb, "[");
+		OSMO_STRBUF_APPEND(sb, osmo_sockaddr_to_str_and_uint, &port, &sockaddr->u.sa);
+		OSMO_STRBUF_PRINTF(sb, "]");
+		if (port)
+			OSMO_STRBUF_PRINTF(sb, ":%u", port);
+		break;
+	default:
+		OSMO_STRBUF_PRINTF(sb, "unsupported family %d", sockaddr->u.sa.sa_family);
+		break;
+	}
+
+	return sb.chars_needed;
+}
+
+/*! string-format a given osmo_sockaddr address into a talloc allocated buffer.
+ * Like osmo_sockaddr_to_str_buf2() but returns a talloc allocated string.
+ *  \param[in] ctx  talloc context to allocate from, e.g. OTC_SELECT.
+ *  \param[in] sockaddr  the osmo_sockaddr to print.
+ *  \return human readable string.
+ */
+char *osmo_sockaddr_to_str_c(void *ctx, const struct osmo_sockaddr *sockaddr)
+{
+	OSMO_NAME_C_IMPL(ctx, 64, "ERROR", osmo_sockaddr_to_str_buf2, sockaddr)
+}
+
+/*! string-format a given osmo_sockaddr address into a user-supplied buffer.
+ * Like osmo_sockaddr_to_str_buf2() but returns buf, or NULL if too short.
  *  \param[in] buf user-supplied output buffer
  *  \param[in] buf_len size of the user-supplied output buffer in bytes
  *  \param[in] sockaddr the osmo_sockaddr to print
@@ -1788,43 +1910,9 @@ const char *osmo_sockaddr_to_str(const struct osmo_sockaddr *sockaddr)
 char *osmo_sockaddr_to_str_buf(char *buf, size_t buf_len,
 			    const struct osmo_sockaddr *sockaddr)
 {
-	uint16_t port = 0;
-	size_t written;
-	if (buf_len < 5)
+	int chars_needed = osmo_sockaddr_to_str_buf2(buf, buf_len, sockaddr);
+	if (chars_needed >= buf_len)
 		return NULL;
-
-	if (!sockaddr)
-		return NULL;
-
-	switch (sockaddr->u.sa.sa_family) {
-	case AF_INET:
-		written = osmo_sockaddr_to_str_and_uint(buf, buf_len, &port, &sockaddr->u.sa);
-		if (written + 1 >= buf_len && port)
-			return NULL;
-		if (port)
-			snprintf(buf + written, buf_len - written, ":%u", port);
-		break;
-	case AF_INET6:
-		buf[0] = '[';
-		written = osmo_sockaddr_to_str_and_uint(buf + 1, buf_len - 1, &port, &sockaddr->u.sa);
-		if (written + 2 >= buf_len)
-			return NULL;
-
-		if (written + 3 >= buf_len && port)
-			return NULL;
-
-		if (port)
-			snprintf(buf + 1 + written, buf_len - written - 1, "]:%u", port);
-		else {
-			buf[written + 1] = ']';
-			buf[written + 2] = 0;
-		}
-		break;
-	default:
-		snprintf(buf, buf_len, "unsupported family %d", sockaddr->u.sa.sa_family);
-		return buf;
-	}
-
 	return buf;
 }
 
