@@ -25,6 +25,8 @@
 
 #define _GNU_SOURCE
 
+#include "../../config.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -276,7 +278,6 @@ static bool proc_name_exists(const char *name, pid_t *res_pid)
 static enum sched_vty_thread_id procname2pid(pid_t *res_pid, const char *str, bool applynow)
 {
 	size_t i, len;
-	char *end;
 	bool is_pid = true;
 
 	if (strcmp(str, "all") == 0) {
@@ -297,12 +298,12 @@ static enum sched_vty_thread_id procname2pid(pid_t *res_pid, const char *str, bo
 		}
 	}
 	if (is_pid) {
-		errno = 0;
-		*res_pid = strtoul(str, &end, 0);
-		if ((errno == ERANGE && *res_pid == ULONG_MAX) || (errno && !*res_pid) ||
-		    str == end) {
+		int64_t val;
+		if (osmo_str_to_int64(&val, str, 0, 0, INT64_MAX))
+			return SCHED_VTY_THREAD_UNKNOWN;
+		*res_pid = (pid_t)val;
+		if (*res_pid != val)
 			return SCHED_VTY_THREAD_UNKNOWN;
-		}
 		if (!applynow || proc_tid_exists(*res_pid))
 			return SCHED_VTY_THREAD_ID;
 		else
@@ -638,8 +639,10 @@ int osmo_cpu_sched_vty_apply_localthread(void)
 		return 0;
 	}
 
+#ifdef HAVE_PTHREAD_GETNAME_NP
 	if (pthread_getname_np(pthread_self(), name, sizeof(name)) == 0)
 		has_name = true;
+#endif
 
 	/* Get latest matching mask for the thread */
 	pthread_mutex_lock(&sched_vty_opts->cpu_affinity_li_mutex);
