@@ -14,10 +14,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include <stdlib.h>
@@ -65,7 +61,7 @@ void vty_out_fsm2(struct vty *vty, const char *prefix, struct osmo_fsm *fsm)
 	if (fsm->event_names) {
 		for (evt_name = fsm->event_names; evt_name->str != NULL; evt_name++) {
 			vty_out(vty, "%s Event %02u (0x%08x): '%s'%s", prefix, evt_name->value,
-				(1 << evt_name->value), evt_name->str, VTY_NEWLINE);
+				(1U << evt_name->value), evt_name->str, VTY_NEWLINE);
 		}
 	} else
 		vty_out(vty, "%s No event names are defined for this FSM! Please fix!%s", prefix, VTY_NEWLINE);
