@@ -16,10 +16,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include <errno.h>
@@ -47,36 +43,39 @@ int osmo_wqueue_bfd_cb(struct osmo_fd *fd, unsigned int what)
 
 	queue = container_of(fd, struct osmo_wqueue, bfd);
 
-	if (what & BSC_FD_READ) {
+	if (what & OSMO_FD_READ) {
 		rc = queue->read_cb(fd);
 		if (rc == -EBADF)
 			goto err_badfd;
 	}
 
-	if (what & BSC_FD_EXCEPT) {
+	if (what & OSMO_FD_EXCEPT) {
 		rc = queue->except_cb(fd);
 		if (rc == -EBADF)
 			goto err_badfd;
 	}
 
-	if (what & BSC_FD_WRITE) {
+	if (what & OSMO_FD_WRITE) {
 		struct msgb *msg;
 
-		fd->when &= ~BSC_FD_WRITE;
+		fd->when &= ~OSMO_FD_WRITE;
 
+		msg = msgb_dequeue_count(&queue->msg_queue, &queue->current_length);
 		/* the queue might have been emptied */
-		if (!llist_empty(&queue->msg_queue)) {
-			--queue->current_length;
-
-			msg = msgb_dequeue(&queue->msg_queue);
+		if (msg) {
 			rc = queue->write_cb(fd, msg);
-			msgb_free(msg);
-
-			if (rc == -EBADF)
+			if (rc == -EBADF) {
+				msgb_free(msg);
 				goto err_badfd;
+			} else if (rc == -EAGAIN) {
+				/* re-enqueue the msgb to the head of the queue */
+				llist_add(&msg->list, &queue->msg_queue);
+				queue->current_length++;
+			} else
+				msgb_free(msg);
 
 			if (!llist_empty(&queue->msg_queue))
-				fd->when |= BSC_FD_WRITE;
+				fd->when |= OSMO_FD_WRITE;
 		}
 	}
 
@@ -100,10 +99,26 @@ void osmo_wqueue_init(struct osmo_wqueue *queue, int max_length)
 	INIT_LLIST_HEAD(&queue->msg_queue);
 }
 
+/*! Enqueue a new \ref msgb into a write queue (without logging full queue events)
+ *  \param[in] queue Write queue to be used
+ *  \param[in] data to-be-enqueued message buffer
+ *  \returns 0 on success; negative on error (MESSAGE NOT FREED IN CASE OF ERROR).
+ */
+int osmo_wqueue_enqueue_quiet(struct osmo_wqueue *queue, struct msgb *data)
+{
+	if (queue->current_length >= queue->max_length)
+		return -ENOSPC;
+
+	msgb_enqueue_count(&queue->msg_queue, data, &queue->current_length);
+	queue->bfd.when |= OSMO_FD_WRITE;
+
+	return 0;
+}
+
 /*! Enqueue a new \ref msgb into a write queue
  *  \param[in] queue Write queue to be used
  *  \param[in] data to-be-enqueued message buffer
- *  \returns 0 on success; negative on error
+ *  \returns 0 on success; negative on error (MESSAGE NOT FREED IN CASE OF ERROR).
  */
 int osmo_wqueue_enqueue(struct osmo_wqueue *queue, struct msgb *data)
 {
@@ -113,11 +128,7 @@ int osmo_wqueue_enqueue(struct osmo_wqueue *queue, struct msgb *data)
 		return -ENOSPC;
 	}
 
-	++queue->current_length;
-	msgb_enqueue(&queue->msg_queue, data);
-	queue->bfd.when |= BSC_FD_WRITE;
-
-	return 0;
+	return osmo_wqueue_enqueue_quiet(queue, data);
 }
 
 /*! Clear a \ref osmo_wqueue
@@ -133,7 +144,7 @@ void osmo_wqueue_clear(struct osmo_wqueue *queue)
 	}
 
 	queue->current_length = 0;
-	queue->bfd.when &= ~BSC_FD_WRITE;
+	queue->bfd.when &= ~OSMO_FD_WRITE;
 }
 
 /*! @} */
