@@ -16,10 +16,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include <osmocom/core/application.h>
@@ -67,12 +63,23 @@ static const uint8_t cm[] = {
 	0x29, 0x47, 0x80, 0x00, 0x00, 0x00, 0x00, 0x80,
 };
 
-static const uint8_t ua[] = {
+static const uint8_t ua_cm[] = {
 	0x01, 0x73, 0x41, 0x05, 0x24, 0x31, 0x03, 0x50,
 	0x18, 0x93, 0x08, 0x29, 0x47, 0x80, 0x00, 0x00,
 	0x00, 0x00, 0x80, 0x2b, 0x2b, 0x2b, 0x2b
 };
 
+static const uint8_t pr[] = {
+	0x06, 0x27, 0x07, 0x03, 0x50, 0x58, 0x92, 0x05,
+	0xf4, 0x44, 0x59, 0xba, 0x63,
+};
+
+static const uint8_t ua_pr[] = {
+	0x01, 0x73, 0x35, 0x06, 0x27, 0x07, 0x03, 0x50,
+	0x58, 0x92, 0x05, 0xf4, 0x44, 0x59, 0xba, 0x63,
+	0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b,
+};
+
 static const uint8_t mm[] = {
 	0x00, 0x0c, 0x00, 0x03, 0x01, 0x01, 0x20, 0x02,
 	0x00, 0x0b, 0x00, 0x03, 0x05, 0x04, 0x0d
@@ -306,8 +313,8 @@ static int dequeue_prim(struct lapdm_entity *le, struct osmo_phsap_prim *pp,
 	/* Take message from queue */
 	rc = lapdm_phsap_dequeue_prim(le, pp);
 
-	fprintf(stderr, "dequeue: got rc %d: %s\n", rc,
-		rc <= 0 ? strerror(-rc) : "-");
+	printf("lapdm_phsap_dequeue_prim(): got rc %d: %s\n",
+	       rc, rc <= 0 ? strerror(-rc) : "-");
 
 	if (rc < 0)
 		return rc;
@@ -317,15 +324,14 @@ static int dequeue_prim(struct lapdm_entity *le, struct osmo_phsap_prim *pp,
 		l3_len = msgb_l3len(pp->oph.msg);
 		l2_header_len -= l3_len;
 	} else
-		fprintf(stderr, "MSGB: L3 is undefined\n");
+		printf("MSGB: L3 is undefined\n");
 
 	if (l2_header_len < 0 || l2_header_len > pp->oph.msg->data_len) {
-		fprintf(stderr,
-			"MSGB inconsistent: data = %p, l2 = %p, l3 = %p, tail = %p\n",
-			pp->oph.msg->data,
-			pp->oph.msg->l2h,
-			pp->oph.msg->l3h,
-			pp->oph.msg->tail);
+		printf("MSGB inconsistent: data = %p, l2 = %p, l3 = %p, tail = %p\n",
+		       pp->oph.msg->data,
+		       pp->oph.msg->l2h,
+		       pp->oph.msg->l3h,
+		       pp->oph.msg->tail);
 		l2_header_len = -1;
 	}
 
@@ -491,7 +497,7 @@ static void test_lapdm_contention_resolution()
 	send_sabm(&bts_to_ms_channel, 0, cm, sizeof(cm));
 	rc = dequeue_prim(&bts_to_ms_channel.lapdm_dcch, &pp, "DCCH");
 	CHECK_RC(rc);
-	OSMO_ASSERT(memcmp(pp.oph.msg->l2h, ua, ARRAY_SIZE(ua)) == 0);
+	OSMO_ASSERT(memcmp(pp.oph.msg->l2h, ua_cm, ARRAY_SIZE(ua_cm)) == 0);
 	msgb_free(pp.oph.msg);
 
 	/* Send SABM MS 2, we must get nothing, due to collision */
@@ -507,7 +513,7 @@ static void test_lapdm_contention_resolution()
 	send_sabm(&bts_to_ms_channel, 0, cm, sizeof(cm));
 	rc = dequeue_prim(&bts_to_ms_channel.lapdm_dcch, &pp, "DCCH");
 	CHECK_RC(rc);
-	OSMO_ASSERT(memcmp(pp.oph.msg->l2h, ua, ARRAY_SIZE(ua)) == 0);
+	OSMO_ASSERT(memcmp(pp.oph.msg->l2h, ua_cm, ARRAY_SIZE(ua_cm)) == 0);
 	msgb_free(pp.oph.msg);
 
 	/* clean up */
@@ -568,10 +574,11 @@ static void lapdm_establish(const uint8_t *est_req, size_t est_req_size)
 	lapdm_channel_set_l1(&bts_to_ms_channel, NULL, &test_state);
 	lapdm_channel_set_l3(&bts_to_ms_channel, bts_to_ms_tx_cb, &test_state);
 
-	/* Send the release request */
+	/* Send the establish request */
 	msg = create_est_req(est_req, est_req_size);
 	rc = lapdm_rslms_recvmsg(msg, &bts_to_ms_channel);
-	fprintf(stderr, "recvmsg: got rc %d: %s\n", rc, rc <= 0 ? strerror(-rc) : "???");
+	printf("lapdm_rslms_recvmsg(): got rc %d: %s\n",
+	       rc, rc <= 0 ? strerror(-rc) : "???");
 	OSMO_ASSERT(rc == 0);
 
 	/* Take message from queue */
@@ -698,7 +705,7 @@ static void test_lapdm_desync()
 
 	rc = dequeue_prim(&bts_to_ms_channel.lapdm_dcch, &pp, "DCCH");
 	CHECK_RC(rc);
-	OSMO_ASSERT(memcmp(pp.oph.msg->l2h, ua, ARRAY_SIZE(ua)) == 0);
+	OSMO_ASSERT(memcmp(pp.oph.msg->l2h, ua_cm, ARRAY_SIZE(ua_cm)) == 0);
 	msgb_free(pp.oph.msg);
 
 	printf("\nSending Classmark Change\n");
@@ -757,6 +764,63 @@ static void test_lapdm_desync()
 	lapdm_channel_exit(&bts_to_ms_channel);
 }
 
+static void test_lapdm_sapi_prio(void)
+{
+	static const uint8_t dl_sabm[] = { 0x0f, 0x3f, 0x01 };
+	struct osmo_phsap_prim pp1, pp2;
+	struct lapdm_channel lc = { };
+	struct msgb *msg;
+	int rc;
+
+	printf("\n=== I test SAPI0/SAPI3 prioritization ===\n\n");
+
+	/* BTS to MS in polling mode */
+	lapdm_channel_init(&lc, LAPDM_MODE_BTS);
+	lapdm_channel_set_flags(&lc, LAPDM_ENT_F_POLLING_ONLY);
+	lapdm_channel_set_l1(&lc, NULL, NULL);
+	lapdm_channel_set_l3(&lc, bts_to_ms_dummy_tx_cb, NULL);
+
+	/* MS establishes a SAPI=0 link by sending func=SABM on SDCCH */
+	printf("MS is establishing a SAPI=0 link\n");
+	send_sabm(&lc, 0, pr, sizeof(pr));
+
+	/* BTS establishes a SAPI=3 link by sending func=SABM on SDCCH */
+	msg = create_est_req(est_req_sdcch_sapi3, sizeof(est_req_sdcch_sapi3));
+	printf("BTS is establishing a SAPI=3 link\n");
+	rc = lapdm_rslms_recvmsg(msg, &lc);
+	OSMO_ASSERT(rc == 0);
+
+	/* Make sure that we get func=UA on SDCCH from the BTS first */
+	rc = dequeue_prim(&lc.lapdm_dcch, &pp1, "DCCH");
+	CHECK_RC(rc);
+
+	/* Contention resolution is completed, we should get func=SABM now */
+	rc = dequeue_prim(&lc.lapdm_dcch, &pp2, "DCCH");
+	CHECK_RC(rc);
+
+	/* Check the actual message payload */
+	rc = memcmp(pp1.oph.msg->l2h, ua_pr, sizeof(ua_pr));
+	printf("Checking the func=UA message: %s\n", rc == 0 ? "OK" : "FAIL");
+	rc = memcmp(pp2.oph.msg->l2h, dl_sabm, sizeof(dl_sabm));
+	printf("Checking the func=SABM message: %s\n", rc == 0 ? "OK" : "FAIL");
+
+	msgb_free(pp1.oph.msg);
+	msgb_free(pp2.oph.msg);
+
+	/* Make sure that the queues are empty now */
+	printf("\nChecking whether the DCCH/SACCH queues are empty\n");
+	rc = dequeue_prim(&lc.lapdm_dcch, &pp1, "DCCH");
+	OSMO_ASSERT(rc < 0);
+	rc = dequeue_prim(&lc.lapdm_dcch, &pp2, "SACCH");
+	OSMO_ASSERT(rc < 0);
+
+	/* Clean up */
+	lapdm_channel_exit(&lc);
+
+	/* Check if exit is idempotent */
+	lapdm_channel_exit(&lc);
+}
+
 int main(int argc, char **argv)
 {
 	void *ctx = talloc_named_const(NULL, 0, "lapd_test");
@@ -773,6 +837,7 @@ int main(int argc, char **argv)
 	test_lapdm_contention_resolution();
 	test_lapdm_establishment();
 	test_lapdm_desync();
+	test_lapdm_sapi_prio();
 
 	printf("Success.\n");
 
