@@ -13,10 +13,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include <stdio.h>
@@ -542,9 +538,9 @@ int main(int argc, char **argv)
 			/* Reserved for libraries */
 			[VTY_TEST_ATTR_UPPER]	= 'X',
 			/* Reserved for global attribues */
-			[VTY_TEST_ATTR_RAFC_DOT]	'.',
-			[VTY_TEST_ATTR_RAFC_EXCL]	'!',
-			[VTY_TEST_ATTR_RAFC_AT]		'@',
+			[VTY_TEST_ATTR_RAFC_DOT]	= '.',
+			[VTY_TEST_ATTR_RAFC_EXCL]	= '!',
+			[VTY_TEST_ATTR_RAFC_AT]		= '@',
 		},
 	};
 
